"""Testing formula 5.36 of EN 1992-1-1:2004."""

import pytest

from blueprints.codes.eurocode.en_1992_1_1_2004.chapter_5_structural_analysis.formula_5_36 import Form5Dot36RelativeAxialForce
from blueprints.validations import LessOrEqualToZeroError, NegativeValueError


class TestForm5Dot36RelativeAxialForce:
    """Validation for formula 5.36 from EN 1992-1-1:2004."""

    def test_evaluation(self) -> None:
        """Tests the evaluation of the result."""
        # Example values
        n_ed = 100000.0
        ac = 2000.0
        fcd = 30.0
        as_ = 100.0
        fyd = 500.0
        n_bal = 0.4

        # Object to test
        formula = Form5Dot36RelativeAxialForce(n_ed=n_ed, ac=ac, fcd=fcd, as_=as_, fyd=fyd, n_bal=n_bal)

        # Expected result, manually calculated
        manually_calculated_result = 0.11627906976  # dimensionless

        assert formula == pytest.approx(expected=manually_calculated_result, rel=1e-4)

    @pytest.mark.parametrize(
        ("n_ed", "ac", "fcd", "as_", "fyd", "n_bal"),
        [
            (-100000.0, 2000.0, 30.0, 100.0, 500.0, 0.4),  # n_ed is negative
            (100000.0, -2000.0, 30.0, 100.0, 500.0, 0.4),  # ac is negative
            (100000.0, 2000.0, -30.0, 100.0, 500.0, 0.4),  # fcd is negative
            (100000.0, 2000.0, 30.0, -100.0, 500.0, 0.4),  # as_ is negative
            (100000.0, 2000.0, 30.0, 100.0, -500.0, 0.4),  # fyd is negative
            (100000.0, 2000.0, 30.0, 100.0, 500.0, -0.4),  # n_bal is negative
            (100000.0, 0.0, 30.0, 100.0, 500.0, 0.4),  # ac is zero
            (100000.0, 2000.0, 0.0, 100.0, 500.0, 0.4),  # fcd is zero
        ],
    )
    def test_raise_error_when_invalid_values_are_given(self, n_ed: float, ac: float, fcd: float, as_: float, fyd: float, n_bal: float) -> None:
        """Test invalid values."""
        with pytest.raises((NegativeValueError, LessOrEqualToZeroError)):
            Form5Dot36RelativeAxialForce(n_ed=n_ed, ac=ac, fcd=fcd, as_=as_, fyd=fyd, n_bal=n_bal)

    @pytest.mark.parametrize(
        ("representation", "expected"),
        [
            (
                "complete",
                r"K_{r} = \min\left(\frac{\left(1 + \frac{A_{s} \cdot f_{yd}}{A_{c} \cdot f_{cd}}\right) - "
                r"\frac{N_{Ed}}{A_{c} \cdot f_{cd}}}{\left(1 + \frac{A_{s} \cdot f_{yd}}{A_{c} \cdot f_{cd}}\right) - n_{bal}}, 1\right) "
                r"= \min\left(\frac{\left(1 + \frac{100.000 \cdot 500.000}{2000.000 \cdot 30.000}\right) - \frac{100000.000}"
                r"{2000.000 \cdot 30.000}}{\left(1 + \frac{100.000 \cdot 500.000}{2000.000 \cdot 30.000}\right) - 0.400}, 1\right) = 0.116 \ -",
            ),
            ("short", r"K_{r} = 0.116 \ -"),
        ],
    )
    def test_latex(self, representation: str, expected: str) -> None:
        """Test the latex representation of the formula."""
        # Example values
        n_ed = 100000.0
        ac = 2000.0
        fcd = 30.0
        as_ = 100.0
        fyd = 500.0
        n_bal = 0.4

        # Object to test
        latex = Form5Dot36RelativeAxialForce(n_ed=n_ed, ac=ac, fcd=fcd, as_=as_, fyd=fyd, n_bal=n_bal).latex()

        actual = {
            "complete": latex.complete,
            "short": latex.short,
        }

        assert expected == actual[representation], f"{representation} representation failed."



"""Formula 6.63 from EN 1992-1-1:2004: Chapter 6 - Ultimate Limit State."""

import numpy as np

from blueprints.codes.eurocode.en_1992_1_1_2004 import EN_1992_1_1_2004
from blueprints.codes.formula import Formula
from blueprints.codes.latex_formula import LatexFormula, latex_replace_symbols
from blueprints.type_alias import MM2, MPA, N
from blueprints.validations import raise_if_less_or_equal_to_zero, raise_if_negative


class Form6Dot63ConcentratedResistanceForce(Formula):
    r"""Class representing formula 6.63 for the calculation of [$F_{Rdu}$]."""

    label = "6.63"
    source_document = EN_1992_1_1_2004

    def __init__(
        self,
        a_c0: MM2,
        a_c1: MM2,
        f_cd: MPA,
    ) -> None:
        r"""[$F_{Rdu}$] Calculation of [$F_{Rdu}$].

        EN 1992-1-1:2004 art.6.7(2) - Formula (6.63)

        Parameters
        ----------
        a_c0 : MM2
            [$A_{c0}$] Loaded area [$mm^2$].
        a_c1 : MM2
            [$A_{c1}$] Maximum design distribution area with a similar shape to [$A_{c0}$] [$mm^2$].
        f_cd : MPA
            [$f_{cd}$] Design compressive strength of concrete [$MPa$].
        """
        super().__init__()
        self.a_c0 = a_c0
        self.a_c1 = a_c1
        self.f_cd = f_cd

    @staticmethod
    def _evaluate(
        a_c0: MM2,
        a_c1: MM2,
        f_cd: MPA,
    ) -> N:
        """Evaluates the formula, for more information see the __init__ method."""
        raise_if_negative(a_c1=a_c1, f_cd=f_cd)
        raise_if_less_or_equal_to_zero(a_c0=a_c0)

        return min(a_c0 * f_cd * np.sqrt(a_c1 / a_c0), 3 * f_cd * a_c0)

    def latex(self, n: int = 3) -> LatexFormula:
        """Returns LatexFormula object for formula 6.63."""
        _equation: str = r"\min \left( A_{c0} \cdot f_{cd} \cdot \sqrt{\frac{A_{c1}}{A_{c0}}}, 3 \cdot f_{cd} \cdot A_{c0} \right)"
        _numeric_equation: str = latex_replace_symbols(
            _equation,
            {
                r"A_{c0}": f"{self.a_c0:.{n}f}",
                r"A_{c1}": f"{self.a_c1:.{n}f}",
                r"f_{cd}": f"{self.f_cd:.{n}f}",
            },
            False,
        )
        return LatexFormula(
            return_symbol=r"F_{Rdu}",
            result=f"{self:.{n}f}",
            equation=_equation,
            numeric_equation=_numeric_equation,
            comparison_operator_label="=",
            unit="N",
        )



"""Steel Strip Profile."""

from collections.abc import Callable
from dataclasses import dataclass
from typing import Self

from matplotlib import pyplot as plt

from blueprints.materials.steel import SteelMaterial
from blueprints.structural_sections.cross_section_rectangle import RectangularCrossSection
from blueprints.structural_sections.steel.steel_cross_sections._steel_cross_section import CombinedSteelCrossSection
from blueprints.structural_sections.steel.steel_cross_sections.plotters.general_steel_plotter import plot_shapes
from blueprints.structural_sections.steel.steel_cross_sections.standard_profiles.strip import Strip
from blueprints.structural_sections.steel.steel_element import SteelElement
from blueprints.type_alias import MM


@dataclass(kw_only=True)
class StripSteelProfile(CombinedSteelCrossSection):
    """Representation of a Steel Strip profile.

    This class is used to create a custom steel strip profile or to create a steel strip profile from a standard profile.
    For standard profiles, use the `from_standard_profile` class method.
    For example,
    ```python
    strip_profile = StripSteelProfile.from_standard_profile(profile=Strip.STRIP160x5, steel_material=SteelMaterial(SteelStrengthClass.S355))
    ```

    Attributes
    ----------
    steel_material : SteelMaterial
        Steel material properties for the profile.
    width : MM
        The width of the strip profile [mm].
    height : MM
        The height (thickness) of the strip profile [mm].
    """

    steel_material: SteelMaterial
    strip_width: MM
    strip_height: MM

    def __post_init__(self) -> None:
        """Initialize the Steel Strip profile."""
        # Nominal thickness is the minimum of width and height
        self.thickness = min(self.strip_width, self.strip_height)

        self.strip = RectangularCrossSection(
            name="Steel Strip",
            width=self.strip_width,
            height=self.strip_height,
            x=0,
            y=0,
        )
        self.elements = [
            SteelElement(
                cross_section=self.strip,
                material=self.steel_material,
                nominal_thickness=self.thickness,
            )
        ]

    @classmethod
    def from_standard_profile(
        cls,
        profile: Strip,
        steel_material: SteelMaterial,
        corrosion: MM = 0,
    ) -> Self:
        """Create a strip profile from a set of standard profiles already defined in Blueprints.

        Parameters
        ----------
        profile : Strip
            Any of the standard strip profiles defined in Blueprints.
        steel_material : SteelMaterial
            Steel material properties for the profile.
        corrosion : MM, optional
            Corrosion thickness per side (default is 0).
        """
        width = profile.width - corrosion * 2
        height = profile.height - corrosion * 2

        if width <= 0 or height <= 0:
            raise ValueError("The profile has fully corroded.")

        name = profile.alias
        if corrosion:
            name += f" (corrosion: {corrosion} mm)"

        return cls(
            steel_material=steel_material,
            strip_width=width,
            strip_height=height,
            name=name,
        )

    def plot(self, plotter: Callable[[CombinedSteelCrossSection], plt.Figure] | None = None, *args, **kwargs) -> plt.Figure:
        """Plot the cross-section. Making use of the standard plotter.

        Parameters
        ----------
        plotter : Callable[CombinedSteelCrossSection, plt.Figure] | None
            The plotter function to use. If None, the default Blueprints plotter for steel sections is used.
        *args
            Additional arguments passed to the plotter.
        **kwargs
            Additional keyword arguments passed to the plotter.
        """
        if plotter is None:
            plotter = plot_shapes
        return plotter(
            self,
            *args,
            **kwargs,
        )



"""Circular cross-section shape."""

from dataclasses import dataclass

from sectionproperties.pre import Geometry
from shapely import Point, Polygon

from blueprints.structural_sections._cross_section import CrossSection
from blueprints.type_alias import MM


@dataclass(frozen=True)
class CircularCrossSection(CrossSection):
    """
    Class to represent a circular cross-section using shapely for geometric calculations.

    Parameters
    ----------
    diameter : MM
        The diameter of the circular cross-section [mm].
    x : MM
        The x-coordinate of the circle's center.
    y : MM
        The y-coordinate of the circle's center.
    name : str
        The name of the rectangular cross-section, default is "Circle".
    """

    diameter: MM
    x: MM
    y: MM
    name: str = "Circle"

    def __post_init__(self) -> None:
        """Post-initialization to validate the diameter."""
        if self.diameter <= 0:
            msg = f"Diameter must be a positive value, but got {self.diameter}"
            raise ValueError(msg)

    @property
    def radius(self) -> MM:
        """
        Calculate the radius of the circular cross-section [mm].

        Returns
        -------
        MM
            The radius of the circle.
        """
        return self.diameter / 2.0

    @property
    def polygon(self) -> Polygon:
        """
        Shapely Polygon representing the circular cross-section.

        Returns
        -------
        Polygon
            The shapely Polygon representing the circle.
        """
        centroid = Point(self.x, self.y)
        return centroid.buffer(self.radius)

    def geometry(
        self,
        mesh_size: MM | None = None,
    ) -> Geometry:
        """Return the geometry of the circular cross-section.

        Properties
        ----------
        mesh_size : MM
            Maximum mesh element area to be used within
            the Geometry-object finite-element mesh. If not provided, a default value will be used.

        """
        if mesh_size is None:
            minimum_mesh_size = 2.0
            mesh_length = max(self.diameter / 20, minimum_mesh_size)
            mesh_size = mesh_length**2

        circular = Geometry(geom=self.polygon)
        circular.create_mesh(mesh_sizes=mesh_size)
        return circular



"""Formula 6.48 from EN 1992-1-1:2004: Chapter 6 - Ultimate Limit State."""

from blueprints.codes.eurocode.en_1992_1_1_2004 import EN_1992_1_1_2004
from blueprints.codes.formula import Formula
from blueprints.codes.latex_formula import LatexFormula, latex_replace_symbols
from blueprints.type_alias import N
from blueprints.validations import raise_if_negative


class Form6Dot48NetAppliedPunchingForce(Formula):
    r"""Class representing formula 6.48 for the calculation of net applied punching force [$V_{Ed,red}$] of slabs and column
    bases without shear reinforcement.
    """

    label = "6.48"
    source_document = EN_1992_1_1_2004

    def __init__(
        self,
        v_ed: N,
        delta_v_ed: N,
    ) -> None:
        r"""[$V_{Ed,red}$] Calculation of net applied punching force of slabs and column bases without shear reinforcement.

        EN 1992-1-1:2004 art.6.4.4(2) - Formula (6.48)

        Parameters
        ----------
        v_ed : N
            [$V_{Ed}$] Applied shear force [$N$].
        delta_v_ed : N
            [$\Delta V_{Ed}$] Net upward force within the control perimeter considered [$N$].
        """
        super().__init__()
        self.v_ed = v_ed
        self.delta_v_ed = delta_v_ed

    @staticmethod
    def _evaluate(
        v_ed: N,
        delta_v_ed: N,
    ) -> N:
        """Evaluates the formula, for more information see the __init__ method."""
        raise_if_negative(v_ed=v_ed, delta_v_ed=delta_v_ed)

        return v_ed - delta_v_ed

    def latex(self, n: int = 3) -> LatexFormula:
        """Returns LatexFormula object for formula 6.48."""
        _equation: str = r"V_{Ed} - \Delta V_{Ed}"
        _numeric_equation: str = latex_replace_symbols(
            _equation,
            {
                r"\Delta V_{Ed}": f"{self.delta_v_ed:.{n}f}",
                r"V_{Ed}": f"{self.v_ed:.{n}f}",
            },
            False,
        )
        return LatexFormula(
            return_symbol=r"V_{Ed,red}",
            result=f"{self._evaluate(self.v_ed, self.delta_v_ed):.{n}f}",
            equation=_equation,
            numeric_equation=_numeric_equation,
            comparison_operator_label="=",
            unit="N",
        )



"""Formula 6.22 from EN 1993-1-1:2005: Chapter 6 - Ultimate Limit State."""

from blueprints.codes.eurocode.en_1993_1_1_2005 import EN_1993_1_1_2005
from blueprints.codes.formula import Formula
from blueprints.codes.latex_formula import LatexFormula, latex_replace_symbols
from blueprints.type_alias import DIMENSIONLESS, MM
from blueprints.validations import raise_if_less_or_equal_to_zero, raise_if_negative


class Form6Dot22CheckShearBucklingResistance(Formula):
    r"""Class representing formula 6.22 for checking shear buckling resistance for webs without intermediate stiffeners."""

    label = "6.22"
    source_document = EN_1993_1_1_2005

    def __init__(
        self,
        h_w: MM,
        t_w: MM,
        epsilon: DIMENSIONLESS,
        eta: DIMENSIONLESS = 1.0,
    ) -> None:
        r"""Check the shear buckling resistance for webs without intermediate stiffeners.

        EN 1993-1-1:2005 art.6.2.6(6) - Formula (6.22)

        Parameters
        ----------
        h_w : MM
            [$h_{w}$] Web height [mm].
        t_w : MM
            [$t_{w}$] Web thickness [mm].
        epsilon : DIMENSIONLESS
            [$\epsilon$] Coefficient depending on $f_y$ [-].
        eta : DIMENSIONLESS, optional
            [$\eta$] See section 5 of EN 1993-1-5, conservatively taken as 1.0 [-].
        """
        super().__init__()
        self.h_w = h_w
        self.t_w = t_w
        self.epsilon = epsilon
        self.eta = eta

    @staticmethod
    def _evaluate(
        h_w: MM,
        t_w: MM,
        epsilon: DIMENSIONLESS,
        eta: DIMENSIONLESS,
    ) -> bool:
        """Evaluates the formula, for more information see the __init__ method."""
        raise_if_less_or_equal_to_zero(t_w=t_w, eta=eta)
        raise_if_negative(h_w=h_w, epsilon=epsilon)

        return (h_w / t_w) > (72 * (epsilon / eta))

    def latex(self, n: int = 3) -> LatexFormula:
        """Returns LatexFormula object for formula 6.22."""
        _equation: str = r"\left( \frac{h_w}{t_w} > 72 \cdot \frac{\epsilon}{\eta} \right)"
        _numeric_equation: str = latex_replace_symbols(
            _equation,
            {
                r"h_w": f"{self.h_w:.{n}f}",
                r"t_w": f"{self.t_w:.{n}f}",
                r"\epsilon": f"{self.epsilon:.{n}f}",
                r"\eta": f"{self.eta:.{n}f}",
            },
            False,
        )
        return LatexFormula(
            return_symbol=r"CHECK",
            result="OK" if self.__bool__() else "\\text{Not OK}",
            equation=_equation,
            numeric_equation=_numeric_equation,
            comparison_operator_label="\\to",
            unit="",
        )



"""Testing formula 9.3 of EN 1992-1-1:2004."""

import pytest

from blueprints.codes.eurocode.en_1992_1_1_2004.chapter_9_detailling_and_specific_rules.formula_9_3 import Form9Dot3ShiftInMomentDiagram
from blueprints.validations import NegativeValueError


class TestForm9Dot3ShiftInMomentDiagram:
    """Validation for formula 9.3 from EN 1992-1-1:2004."""

    def test_evaluation(self) -> None:
        """Test the evaluation of the result."""
        # Example values
        v_ed = -200  # kN
        a_l = 200  # mm
        z = 250  # mm
        n_ed = 500  # kN
        form_9_3 = Form9Dot3ShiftInMomentDiagram(
            v_ed=v_ed,
            a_l=a_l,
            z=z,
            n_ed=n_ed,
        )

        # Expected result, manually calculated
        manually_calculated_result = 660

        assert form_9_3 == pytest.approx(expected=manually_calculated_result, rel=1e-4)

    def test_raise_error_when_negative_z_is_given(self) -> None:
        """Test if error is raised when z is negative."""
        # Example values
        v_ed = -200  # kN
        a_l = 200  # mm
        z = -250  # mm
        n_ed = 500  # kN

        with pytest.raises(NegativeValueError):
            Form9Dot3ShiftInMomentDiagram(
                v_ed=v_ed,
                a_l=a_l,
                z=z,
                n_ed=n_ed,
            )

    def test_raise_error_when_negative_a_l_is_given(self) -> None:
        """Test if error is raised when a_l is negative."""
        # Example values
        v_ed = -200  # kN
        a_l = -200  # mm
        z = 250  # mm
        n_ed = 500  # kN

        with pytest.raises(NegativeValueError):
            Form9Dot3ShiftInMomentDiagram(
                v_ed=v_ed,
                a_l=a_l,
                z=z,
                n_ed=n_ed,
            )

    @pytest.mark.parametrize(
        ("representation", "expected"),
        [
            (
                "complete",
                r"F_E = |V_{Ed}| \cdot a_l / z + N_{Ed} = |-200.00| \cdot 200.00 / 250.00 + 500.00 = 660.00",
            ),
            ("short", r"F_E = 660.00"),
        ],
    )
    def test_latex(self, representation: str, expected: str) -> None:
        """Test the latex representation of the formula."""
        # Example values
        v_ed = -200  # kN
        a_l = 200  # mm
        z = 250  # mm
        n_ed = 500  # kN

        # Object to test
        form_9_3_latex = Form9Dot3ShiftInMomentDiagram(
            v_ed=v_ed,
            a_l=a_l,
            z=z,
            n_ed=n_ed,
        ).latex()

        actual = {"complete": form_9_3_latex.complete, "short": form_9_3_latex.short}

        assert actual[representation] == expected, f"{representation} representation failed."



"""Testing formula 7.5 from EN 1995-1-1:2004."""

from contextlib import AbstractContextManager
from contextlib import nullcontext as does_not_raise

import pytest

from blueprints.codes.eurocode.en_1995_1_1_2004.chapter_7_serviceability_limit_states.formula_7_5 import Form7Dot5NaturalFrequency
from blueprints.validations import LessOrEqualToZeroError, NegativeValueError


class TestForm7Dot5NaturalFrequency:
    """Validation for formula 7.5 from EN 1995-1-1:2004."""

    @pytest.mark.parametrize(
        ("length", "ei_l", "m", "expected_result"),
        [(5.3, 5.81e6, 226, 8.9661), (1.25, 0, 1, 0.00)],
        ids=[
            "result 1",
            "result 2",
        ],
    )
    def test_evaluation(self, length: float, ei_l: float, m: float, expected_result: float) -> None:
        """Test the evaluation of the result."""
        form_7_5 = Form7Dot5NaturalFrequency(length=length, ei_l=ei_l, m=m)
        assert form_7_5 == pytest.approx(expected=expected_result, rel=1e-4)

    @pytest.mark.parametrize(
        ("length", "ei_l", "m", "expectation"),
        [
            (5.3, 5.81e6, 226, does_not_raise()),
            (1.25, 0, 1, does_not_raise()),
            (5.3, 5.8e06, 0, pytest.raises(LessOrEqualToZeroError)),
            (0, 5.8e06, 20, pytest.raises(LessOrEqualToZeroError)),
            (1, -5.8e06, 20, pytest.raises(NegativeValueError)),
        ],
        ids=["decimal f1", "integer f1", "zero_m", "zero_l", "negative_ei"],
    )
    def test_raise_error_incorrect_args(self, length: float, ei_l: float, m: float, expectation: AbstractContextManager) -> None:
        """Test the evaluation of the result."""
        with expectation:
            assert Form7Dot5NaturalFrequency(length=length, ei_l=ei_l, m=m) is not None

    @pytest.mark.parametrize(
        ("length", "ei_l", "m", "rep_short", "rep_long"),
        [
            (
                5.30,  # M
                5.81e6,  # NM2_M
                226,  # KG_M2
                r"f_{1} = 8.97",
                r"f_{1} = \frac{\pi}{2 \cdot l^{2}} \cdot \sqrt{\frac{(EI)_{l}}{m}} = \frac{\pi}{2 \cdot 5.30^{2}} \cdot \sqrt{\frac{5810000.00}{226.00}} = 8.97",  # noqa: E501
            ),
            (
                1.25,  # M
                0,  # NM2_M
                1,  # KG_M2
                r"f_{1} = 0.00",
                r"f_{1} = \frac{\pi}{2 \cdot l^{2}} \cdot \sqrt{\frac{(EI)_{l}}{m}} = \frac{\pi}{2 \cdot 1.25^{2}} \cdot \sqrt{\frac{0.00}{1.00}} = 0.00",  # noqa: E501
            ),
        ],
        ids=["test with decimal outcome", "test with outcome=0"],
    )
    def test_latex(self, length: float, ei_l: float, m: float, rep_short: str, rep_long: str) -> None:
        """Text the latex representation of the formula."""
        # Objects to test

        # arrange
        # act
        form_7_5_latex = Form7Dot5NaturalFrequency(length=length, ei_l=ei_l, m=m).latex()

        # assert
        assert form_7_5_latex.complete == rep_long
        assert form_7_5_latex.short == rep_short



"""Formula 6.21 from EN 1992-1-1:2004: Chapter 6 - Ultimate Limit State."""

from blueprints.codes.eurocode.en_1992_1_1_2004 import EN_1992_1_1_2004
from blueprints.codes.formula import Formula
from blueprints.codes.latex_formula import LatexFormula
from blueprints.math_helpers import cot
from blueprints.type_alias import DEG, MM, MM2, MPA
from blueprints.validations import raise_if_less_or_equal_to_zero, raise_if_negative


class Form6Dot21CheckTransverseReinforcement(Formula):
    r"""Class representing formula 6.21 for checking transverse reinforcement per unit length."""

    label = "6.21"
    source_document = EN_1992_1_1_2004

    def __init__(
        self,
        a_sf: MM2,
        f_yd: MPA,
        s_f: MM,
        v_ed: MPA,
        h_f: MM,
        theta_f: DEG,
    ) -> None:
        r"""Check the transverse reinforcement per unit length.

        EN 1992-1-1:2004 art.6.2.4(4) - Formula (6.21)

        Parameters
        ----------
        a_sf : MM2
            [$A_{sf}$] Area of transverse reinforcement per unit length [$mm^2$].
        f_yd : MPA
            [$f_{yd}$] Design yield strength of reinforcement [$MPa$].
        s_f : MM
            [$s_{f}$] Spacing of transverse reinforcement [$mm$].
        v_ed : MPA
            [$v_{Ed}$] Design shear stress [$MPa$].
        h_f : MM
            [$h_{f}$] Thickness of flange at the junctions [$mm$].
        theta_f : DEG
            [$\theta_{f}$] Angle of the compression strut [$degrees$].
        """
        super().__init__()
        self.a_sf = a_sf
        self.f_yd = f_yd
        self.s_f = s_f
        self.v_ed = v_ed
        self.h_f = h_f
        self.theta_f = theta_f

    @staticmethod
    def _evaluate(
        a_sf: MM2,
        f_yd: MPA,
        s_f: MM,
        v_ed: MPA,
        h_f: MM,
        theta_f: DEG,
    ) -> bool:
        """Evaluates the formula, for more information see the __init__ method."""
        raise_if_negative(a_sf=a_sf, f_yd=f_yd, v_ed=v_ed, h_f=h_f, theta_f=theta_f)
        denominator_rhs = cot(theta_f)
        raise_if_less_or_equal_to_zero(s_f=s_f, denominator_rhs=denominator_rhs)

        return (a_sf * f_yd / s_f) >= (v_ed * h_f / cot(theta_f))

    def latex(self, n: int = 3) -> LatexFormula:
        """Returns LatexFormula object for formula 6.21."""
        return LatexFormula(
            return_symbol=r"CHECK",
            result="OK" if self.__bool__() else "\\text{Not OK}",
            equation=r"\left( \frac{A_{sf} \cdot f_{yd}}{s_{f}} \geq \frac{v_{Ed} \cdot h_{f}}{\cot(\theta_{f})} \right)",
            numeric_equation=rf"\left( \frac{{{self.a_sf:.{n}f} \cdot {self.f_yd:.{n}f}}}{{{self.s_f:.{n}f}}} \geq \frac{{{self.v_ed:.{n}f} "
            rf"\cdot {self.h_f:.{n}f}}}{{\cot({self.theta_f:.{n}f})}} \right)",
            comparison_operator_label="\\to",
            unit="",
        )



"""Line module."""

from itertools import pairwise
from typing import Literal, Self

import numpy as np
from shapely import Point

from blueprints.geometry.operations import CoordinateSystemOptions, calculate_rotation_angle
from blueprints.type_alias import DEG
from blueprints.unit_conversion import RAD_TO_DEG


class Line:
    """Represents a line in a 3D modelling space.

    Parameters
    ----------
    start_point : Point
        Starting point
    end_point : Point
        End point
    """

    id: int = 0

    def __init__(self, start_point: Point, end_point: Point) -> None:
        """Initialize the line."""
        self._start_point = start_point
        self._end_point = end_point
        self._validate_points()
        Line.id += 1

    @property
    def start_point(self) -> Point:
        """Return the start point."""
        return self._start_point

    @start_point.setter
    def start_point(self, value: Point) -> None:
        """Set the start point."""
        self._start_point = value
        self._validate_points()

    @property
    def end_point(self) -> Point:
        """Return the end point."""
        return self._end_point

    @end_point.setter
    def end_point(self, value: Point) -> None:
        """Set the end point."""
        self._end_point = value
        self._validate_points()

    @property
    def _start(self) -> np.ndarray:
        """Return the start point as a numpy array."""
        return np.array(self._start_point.coords)

    @property
    def _end(self) -> np.ndarray:
        """Return the end point as a numpy array."""
        return np.array(self._end_point.coords)

    def _validate_points(self) -> None:
        """Validate if the points are different."""
        # Check if start and end point are the same
        if list(self.start_point.coords) == list(self.end_point.coords):
            msg = f"Start and end point can't be equal. {self.start_point=} | {self.end_point=}"
            raise ValueError(msg)

        # if points have no z value, then declare zero as default
        if not self.start_point.has_z:
            self.start_point = Point(self.start_point.x, self.start_point.y, 0.0)
        if not self.end_point.has_z:
            self.end_point = Point(self.end_point.x, self.end_point.y, 0.0)

    @property
    def midpoint(self) -> Point:
        """Midpoint of the line."""
        return Point((self._start + self._end) / 2)

    @property
    def delta_x(self) -> float:
        """Difference in X-coordinate between starting and end point (X end - X start)."""
        return self.end_point.x - self.start_point.x

    @property
    def delta_y(self) -> float:
        """Difference in Y-coordinate between starting and end point (Y end - Y start)."""
        return self.end_point.y - self.start_point.y

    @property
    def delta_z(self) -> float:
        """Difference in Z-coordinate between starting and end point (Z end - Z start)."""
        return self.end_point.z - self.start_point.z

    @property
    def length(self) -> float:
        """Return the total length of the line."""
        return float(np.linalg.norm(self._end - self._start))

    def angle(self, coordinate_system: CoordinateSystemOptions = CoordinateSystemOptions.XY) -> DEG:
        """
        Calculates rotation of the end point in relation to the start point in a given plane/coordinate system [deg].

        - The rotation is calculated in the given plane in relation to the start point.
        - The rotation is calculated in the range [0, 2*pi].
        - The rotation is calculated in the counter-clockwise direction.

        Parameters
        ----------
        coordinate_system: CoordinateSystemOptions
            Desired plane that will be used as reference to calculate the rotation. Standard is XY-plane.

        Returns
        -------
        DEG
            rotation of end point relative to the start point in degrees in the given plane [deg].
        """
        return (
            calculate_rotation_angle(
                start_point=self.start_point,
                end_point=self.end_point,
                coordinate_system=coordinate_system,
            )
            * RAD_TO_DEG
        )

    @property
    def unit_vector(self) -> np.ndarray:
        """Return the unit vector of the line."""
        return (self._end - self._start) / self.length

    def get_internal_point(self, distance: float, reference: Literal["start", "end"] = "start") -> Point:
        """Return an internal point within the line in a given distance from the reference point.

        Parameters
        ----------
        distance : float
            Distance from the given reference point following the axis of the line
        reference: Literal["start", "end"]
            Reference point in the line where given distance is declared. Default -> "start"

        Returns
        -------
        Point
            Internal point within the line in a given distance from the reference point.

        Raises
        ------
        ValueError
            If the distance is greater than the total length of the line.
            If the distance is a negative number.
        """
        if distance > self.length:
            msg = f"Distance must be equal or less than total length of the line. Length={self.length} | Distance={distance}"
            raise ValueError(msg)
        if distance < 0:
            msg = "Given Distance must be a positive number."
            raise ValueError(msg)

        match reference.lower():
            case "start":
                internal_point = self._start + distance * self.unit_vector
            case "end":
                internal_point = self._end - distance * self.unit_vector
            case _:
                msg = f"'{reference}' is an invalid input for 'reference_point', use 'start' or 'end'."
                raise ValueError(msg)
        return Point(internal_point)

    def adjust_length(self, distance: float, direction: Literal["start", "end"] = "end") -> Self:
        """Extends or shortens the line in a given direction. The end of the line is the default direction.

        Parameters
        ----------
        distance : float
            Distance to extend or shorten the line. Positive number extends the line, negative number shortens the line.
        direction: Literal["start", "end"]
            Given direction where the line needs to be extended. Default towards the end of the line.

        Returns
        -------
        Line
            The new line with the adjusted length.
        """
        if distance < 0 and abs(distance) >= self.length:
            raise ValueError("When shortening the line, the absolute value of the extra length must be less than the total length of the line.")

        match direction.lower():
            case "end":
                new_point = self._end + distance * self.unit_vector
                self.end_point = Point(new_point)
            case "start":
                new_point = self._start - distance * self.unit_vector
                self.start_point = Point(new_point)
            case _:
                msg = "Invalid input for 'direction', use 'start' or 'end'."
                raise ValueError(msg)
        return self

    def get_evenly_spaced_points(self, n: int = 2) -> list[Point]:
        """Return a list of evenly spaced internal points of the line from start to end point with an n number of desired points.

        Parameters
        ----------
        n : int
            Total number of internal points desired. A minimum of 2 points are required.
        """
        if not isinstance(n, int):
            msg = "n must be an integer"
            raise TypeError(msg)
        if n < 2:
            msg = "n must be equal or greater than 2"
            raise ValueError(msg)

        # Create a list of evenly spaced points
        evenly_spaced_points = np.linspace(start=0, stop=self.length, num=n, endpoint=True)

        return [Point(self._start + distance * self.unit_vector) for distance in evenly_spaced_points]

    def divide_into_n_lines(self, n: int = 2) -> list["Line"]:
        """Return a list of evenly divided lines.

        Parameters
        ----------
        n : int
            Total number of lines desired. A minimum of 2 lines are required.
        """
        if not isinstance(n, int):
            msg = "n must be an integer"
            raise TypeError(msg)
        if n < 2:
            msg = "n must be equal or greater than 2"
            raise ValueError(msg)

        evenly_spaced_points = self.get_evenly_spaced_points(n + 1)
        return [Line(start_point=point_1, end_point=point_2) for point_1, point_2 in pairwise(evenly_spaced_points)]

    def __eq__(self, other: object) -> bool:
        """Return True if the lines are equal."""
        if not isinstance(other, Line):
            raise NotImplementedError("Line can only be compared to other Line object")
        return np.allclose(self._start, other._start) and np.allclose(self._end, other._end)

    def __repr__(self) -> str:
        """Return the representation of the line."""
        return f"Line({self.start_point}, {self.end_point})"



"""Testing formula 5.26 of EN 1992-1-1:2004."""

import pytest

from blueprints.codes.eurocode.en_1992_1_1_2004.chapter_5_structural_analysis.formula_5_26 import Form5Dot26FactorKc, Form5Dot26FactorKs
from blueprints.validations import LessOrEqualToZeroError


class TestForm5Dot26FactorKs:
    """Validation for formula 5.26 (Factor Ks) from EN 1992-1-1:2004."""

    def test_evaluation(self) -> None:
        """Test the evaluation of the result."""
        # Example values
        rho = 0.02  # -

        # Object to test
        form_5_26_ks = Form5Dot26FactorKs(rho=rho)

        # Expected result, manually calculated
        manually_calculated_result = 0.0

        assert form_5_26_ks == pytest.approx(expected=manually_calculated_result, rel=1e-4)

    def test_raise_error_when_rho_is_less_than_0_01(self) -> None:
        """Test a value for rho less than 0.01."""
        # Example values
        rho = 0.009

        with pytest.raises(ValueError):
            Form5Dot26FactorKs(rho=rho)

    @pytest.mark.parametrize(
        ("representation", "expected_result"),
        [
            ("complete", r"K_s = 0 = 0 = 0.000"),
            ("short", "K_s = 0.000"),
        ],
    )
    def test_latex(self, representation: str, expected_result: str) -> None:
        """Test the latex representation of the formula."""
        # Example values
        rho = 0.02  # -

        # Object to test
        form_5_26_ks_latex = Form5Dot26FactorKs(rho=rho).latex()

        actual = {
            "complete": form_5_26_ks_latex.complete,
            "short": form_5_26_ks_latex.short,
        }

        assert actual[representation] == expected_result, f"{representation} representation failed."


class TestForm5Dot26FactorKc:
    """Validation for formula 5.26 (Factor Kc) from EN 1992-1-1:2004."""

    def test_evaluation(self) -> None:
        """Test the evaluation of the result."""
        # Example values
        phi_ef = 2.0  # -
        rho = 0.02  # -

        # Object to test
        form_5_26_kc = Form5Dot26FactorKc(phi_ef=phi_ef, rho=rho)

        # Expected result, manually calculated
        manually_calculated_result = 0.15

        assert form_5_26_kc == pytest.approx(expected=manually_calculated_result, rel=1e-4)

    def test_raise_error_when_rho_is_less_or_equal_to_0_01(self) -> None:
        """Test a value for rho less than or equal to 0.01."""
        # Example values
        phi_ef = 2.0
        rho = 0.01

        with pytest.raises(ValueError):
            Form5Dot26FactorKc(phi_ef=phi_ef, rho=rho)

    def test_raise_error_when_phi_ef_is_less_or_equal_to_zero(self) -> None:
        """Test a value for phi_ef less than or equal to zero."""
        # Example values
        phi_ef = 0.0
        rho = 0.02

        with pytest.raises(LessOrEqualToZeroError):
            Form5Dot26FactorKc(phi_ef=phi_ef, rho=rho)

    @pytest.mark.parametrize(
        ("representation", "expected_result"),
        [
            ("complete", r"K_c = \frac{0.3}{1 + 0.5 \cdot \phi_{ef}} = \frac{0.3}{1 + 0.5 \cdot 2.000} = 0.150"),
            ("short", "K_c = 0.150"),
        ],
    )
    def test_latex(self, representation: str, expected_result: str) -> None:
        """Test the latex representation of the formula."""
        # Example values
        phi_ef = 2.0  # -
        rho = 0.02  # -

        # Object to test
        form_5_26_kc_latex = Form5Dot26FactorKc(phi_ef=phi_ef, rho=rho).latex()

        actual = {
            "complete": form_5_26_kc_latex.complete,
            "short": form_5_26_kc_latex.short,
        }

        assert actual[representation] == expected_result, f"{representation} representation failed."



"""Formula 3.26 from EN 1992-1-1:2004: Chapter 3 - Materials."""

from blueprints.codes.eurocode.en_1992_1_1_2004 import EN_1992_1_1_2004
from blueprints.codes.formula import Formula
from blueprints.codes.latex_formula import LatexFormula
from blueprints.type_alias import DIMENSIONLESS, MPA


class Form3Dot26IncreasedStrainAtMaxStrength(Formula):
    """Class representing formula 3.26 for the calculation of the increased strain at the maximum strength due to enclosed concrete."""

    label = "3.26"
    source_document = EN_1992_1_1_2004

    def __init__(
        self,
        f_ck: MPA,
        f_ck_c: MPA,
        epsilon_c2: DIMENSIONLESS,
    ) -> None:
        r"""[$\epsilon_{c2,c}$] Increased strain at the maximum strength due to enclosed concrete. [$-$].

        EN 1992-1-1:2004 art.3.1.9(2) - Formula (3.26)

        Parameters
        ----------
        f_ck : MPA
            [$f_{ck}$] Characteristic compressive strength [$MPa$]
        f_ck_c : MPA
            [$f_{ck,c}$] Increased characteristic compressive strength due to enclosed concrete [$MPa$].
            See classes Form3Dot24IncreasedCharacteristicCompressiveStrength and/or Form3Dot25IncreasedCharacteristicCompressiveStrength
        epsilon_c2 : DIMENSIONLESS
            [$\epsilon_{c2}$] Strain at maximum strength [$-$]

        Returns
        -------
        None
        """
        super().__init__()
        self.f_ck = f_ck
        self.f_ck_c = f_ck_c
        self.epsilon_c2 = epsilon_c2

    @staticmethod
    def _evaluate(
        f_ck: MPA,
        f_ck_c: MPA,
        epsilon_c2: DIMENSIONLESS,
    ) -> float:
        """Evaluates the formula, for more information see the __init__ method."""
        if f_ck < 0:
            raise ValueError(f"Invalid f_ck: {f_ck}. f_ck cannot be negative")
        if f_ck_c < 0:
            raise ValueError(f"Invalid f_ck_c: {f_ck_c}. f_ck_c cannot be negative")
        return epsilon_c2 * (f_ck_c / f_ck) ** 2

    def latex(self, n: int = 3) -> LatexFormula:
        """Returns LatexFormula object for formula 3.26."""
        return LatexFormula(
            return_symbol=r"\epsilon_{c2,c}",
            result=f"{self:.{n}f}",
            equation=r"\epsilon_{c2} \cdot ( f_{ck,c} / f_{ck} )^2",
            numeric_equation=rf"{self.epsilon_c2:.{n}f} \cdot ( {self.f_ck_c:.{n}f} / {self.f_ck:.{n}f} )^2",
            comparison_operator_label="=",
        )



"""Test suite for the CHSSteelProfile class."""

from unittest.mock import MagicMock

import matplotlib.pyplot as plt
import pytest

from blueprints.codes.eurocode.en_1993_1_1_2005.chapter_3_materials.table_3_1 import SteelStrengthClass
from blueprints.materials.steel import SteelMaterial
from blueprints.structural_sections.steel.steel_cross_sections.chs_profile import CHSSteelProfile
from blueprints.structural_sections.steel.steel_cross_sections.standard_profiles.chs import CHS


class TestCHSSteelProfile:
    """Test suite for CHSSteelProfile."""

    def test_name(self, chs_profile: CHSSteelProfile) -> None:
        """Test the name of the CHS profile."""
        expected_name: str = "CHS 508x16"
        assert chs_profile.name == expected_name

    def test_code(self, chs_profile: CHSSteelProfile) -> None:
        """Test the code of the CHS profile."""
        expected_alias: str = "CHS 508x16"
        assert chs_profile.name == expected_alias

    def test_steel_volume_per_meter(self, chs_profile: CHSSteelProfile) -> None:
        """Test the steel volume per meter."""
        expected_volume: float = 2.47e-2  # m³/m
        assert pytest.approx(chs_profile.volume_per_meter, rel=1e-2) == expected_volume

    def test_steel_weight_per_meter(self, chs_profile: CHSSteelProfile) -> None:
        """Test the steel weight per meter."""
        expected_weight: float = 2.47e-2 * 7850  # kg/m
        assert pytest.approx(chs_profile.weight_per_meter, rel=1e-2) == expected_weight

    @pytest.mark.slow
    def test_plot(self, chs_profile: CHSSteelProfile) -> None:
        """Test the plotting of the CHS profile shapes."""
        fig = chs_profile.plot(show=False)
        assert fig is not None
        assert isinstance(fig, plt.Figure)

    def test_plot_mocked(self, chs_profile: CHSSteelProfile, mock_section_properties: MagicMock) -> None:  # noqa: ARG002
        """Test the plotting of the CHS profile shapes with mocked section properties."""
        fig = chs_profile.plot(show=False)
        assert fig is not None
        assert isinstance(fig, plt.Figure)

    def test_get_profile_with_corrosion(self) -> None:
        """Test the CHS profile with 16 mm corrosion applied."""
        # Ensure the profile raises an error if fully corroded
        with pytest.raises(ValueError, match="The profile has fully corroded."):
            CHSSteelProfile.from_standard_profile(
                profile=CHS.CHS508x16,
                steel_material=SteelMaterial(SteelStrengthClass.S355),
                corrosion_outside=5,  # mm
                corrosion_inside=11,  # mm
            )

    def test_corrosion_in_name(self) -> None:
        """Test that the name includes corrosion information."""
        chs_profile_with_corrosion = CHSSteelProfile.from_standard_profile(
            profile=CHS.CHS508x16,
            steel_material=SteelMaterial(SteelStrengthClass.S355),
            corrosion_outside=1,  # mm
            corrosion_inside=2,  # mm
        )
        expected_name_with_corrosion = "CHS 508x16 (corrosion in: 2 mm, out: 1 mm)"
        assert chs_profile_with_corrosion.name == expected_name_with_corrosion



"""Testing formula 4.3N of EN 1992-1-1:2004."""

import pytest

from blueprints.codes.eurocode.en_1992_1_1_2004.chapter_4_durability_and_cover.formula_4_3n import Form4Dot3NCheckExecutionTolerances
from blueprints.validations import LessOrEqualToZeroError, NegativeValueError


class TestForm4Dot3NCheckExecutionTolerances:
    """Validation for formula 4.3N from EN 1992-1-1:2004."""

    def test_evaluation(self) -> None:
        """Tests the evaluation of the result."""
        # Example values
        delta_cdev = 7.0

        # Object to test
        formula = Form4Dot3NCheckExecutionTolerances(delta_cdev=delta_cdev)

        # Expected result, manually calculated
        expected_result = True

        assert formula == expected_result

    @pytest.mark.parametrize(
        ("delta_cdev"),
        [
            (-1.0),  # delta_cdev
            (0.0),  # delta_cdev is 0
        ],
    )
    def test_raise_error_when_invalid_values_are_given(self, delta_cdev: float) -> None:
        """Test invalid values."""
        with pytest.raises((NegativeValueError, LessOrEqualToZeroError)):
            Form4Dot3NCheckExecutionTolerances(delta_cdev=delta_cdev)

    @pytest.mark.parametrize(
        ("representation", "expected"),
        [
            (
                "complete",
                r"CHECK \to 5 \leq \Delta c_{dev} \leq 10 \to 5 \leq 7.000 \leq 10 \to OK",
            ),
            ("short", r"CHECK \to OK"),
        ],
    )
    def test_latex(self, representation: str, expected: str) -> None:
        """Test the latex representation of the formula."""
        # Example values
        delta_cdev = 7.0

        # Object to test
        latex = Form4Dot3NCheckExecutionTolerances(delta_cdev=delta_cdev).latex()

        actual = {
            "complete": latex.complete,
            "short": latex.short,
        }

        assert expected == actual[representation], f"{representation} representation failed."



"""Formula 3.17 from EN 1992-1-1:2004: Chapter 3 - Materials."""

from blueprints.codes.eurocode.en_1992_1_1_2004 import EN_1992_1_1_2004
from blueprints.codes.formula import Formula
from blueprints.codes.latex_formula import LatexFormula
from blueprints.type_alias import DIMENSIONLESS, MPA


class Form3Dot17CompressiveStressConcrete(Formula):
    """Class representing formula 3.17 for the calculation of compressive stress in concrete using stress-strain diagram of figure 3.3."""

    label = "3.17"
    source_document = EN_1992_1_1_2004

    def __init__(
        self,
        f_cd: MPA,
        epsilon_c: DIMENSIONLESS,
        epsilon_c2: DIMENSIONLESS,
        n: DIMENSIONLESS,
    ) -> None:
        r"""[$\sigma_c$] Compressive stress in concrete using stress-strain diagram of figure 3.3 [$MPa$].

        EN 1992-1-1:2004 art.3.1.7(1) - Formula (3.17)

        Parameters
        ----------
        f_cd : MPA
            [$f_{cd}$] Design value compressive strength concrete [$MPa$].
        epsilon_c : DIMENSIONLESS
            [$\epsilon_c$] Strain in concrete [$-$].
        epsilon_c2 : DIMENSIONLESS
            [$\epsilon_{c2}$] Strain in concrete when reaching maximum strength following table 3.1 [$-$].
        n : DIMENSIONLESS
            Exponent following table 3.1.

        Returns
        -------
        None
        """
        super().__init__()
        self.f_cd = f_cd
        self.epsilon_c = epsilon_c
        self.epsilon_c2 = epsilon_c2
        self.n = n

    @staticmethod
    def _evaluate(
        f_cd: MPA,
        epsilon_c: DIMENSIONLESS,
        epsilon_c2: DIMENSIONLESS,
        n: DIMENSIONLESS,
    ) -> MPA:
        """Evaluates the formula, for more information see the __init__ method."""
        if f_cd < 0:
            raise ValueError(f"Invalid f_cd: {f_cd}. f_cd cannot be negative")
        if epsilon_c < 0:
            raise ValueError(f"Invalid epsilon_c: {epsilon_c}. epsilon_c cannot be negative")
        if epsilon_c > epsilon_c2:
            raise ValueError(f"epsilon_c: {epsilon_c} > epsilon_c2: {epsilon_c2}. Try using Form3Dot18CompressiveStressConcrete class.")
        return f_cd * (1 - (1 - (epsilon_c / epsilon_c2)) ** n)

    def latex(self, n: int = 3) -> LatexFormula:
        """Returns LatexFormula object for formula 3.17."""
        return LatexFormula(
            return_symbol=r"\sigma_c",
            result=f"{self:.{n}f}",
            equation=r"f_{cd} \cdot \left[ 1 - \left( 1 - \frac{\epsilon_c}{\epsilon_{c2}} \right)^n \right]",
            numeric_equation=(
                rf"{self.f_cd:.{n}f} \cdot \left[ 1 - \left( 1 - \frac{{{self.epsilon_c:.{n}f}}}"
                rf"{{{self.epsilon_c2:.{n}f}}} \right)^{{{self.n:.{n}f}}} \right]"
            ),
            comparison_operator_label="=",
        )



"""Formula 3.6 from EN 1992-1-1:2004: Chapter 3 - Materials."""

from blueprints.codes.eurocode.en_1992_1_1_2004 import EN_1992_1_1_2004
from blueprints.codes.formula import Formula
from blueprints.codes.latex_formula import LatexFormula
from blueprints.type_alias import DIMENSIONLESS, MPA


class Form3Dot6CreepDeformationOfConcrete(Formula):
    """Class representing formula 3.6 for the calculation of creep deformation of concrete."""

    label = "3.6"
    source_document = EN_1992_1_1_2004

    def __init__(
        self,
        phi_inf_t0: DIMENSIONLESS,
        sigma_c: MPA,
        e_c: MPA,
    ) -> None:
        r"""εcc(∞,t0) Creep deformation of concrete at the time t = ∞ for a constant concrete
        compressive stress [$\sigma_c$] applied at time [$t_0$] [$-$].

        EN 1992-1-1:2004 art.3.1.4(3) - Formula (3.6)

        Parameters
        ----------
        phi_inf_t0 : DIMENSIONLESS
            [$\varphi(\infty, t_0)$] Creep coefficient if high accuracy is not required use figure 3.1 else use appendix B [$-$].
        sigma_c : MPA
            [$\sigma_c$] Concrete compressive stress [$MPa$].
        e_c : MPA
            [$E_c$] tangent modulus = 1.05 * [$E_{cm}$]. According to art.3.1.4(2) [$MPa$].

        Returns
        -------
        None
        """
        super().__init__()
        self.phi_inf_t0 = phi_inf_t0
        self.sigma_c = sigma_c
        self.e_c = e_c

    @staticmethod
    def _evaluate(
        phi_inf_t0: DIMENSIONLESS,
        sigma_c: MPA,
        e_c: MPA,
    ) -> DIMENSIONLESS:
        """Evaluates the formula, for more information see the __init__ method."""
        if phi_inf_t0 < 0:
            raise ValueError(f"Negative phi_inf_t0: {phi_inf_t0}. phi_inf_t0 cannot be negative")
        if sigma_c < 0:
            raise ValueError(f"Negative sigma_c: {sigma_c}. sigma_c cannot be negative")
        if e_c < 0:
            raise ValueError(f"Negative e_c: {e_c}. e_c cannot be negative")
        return phi_inf_t0 * sigma_c / e_c

    def latex(self, n: int = 3) -> LatexFormula:
        """Returns LatexFormula object for formula 3.6."""
        return LatexFormula(
            return_symbol=r"\epsilon_{cc}(\infty, t_0)",
            result=f"{self:.{n}f}",
            equation=r"\varphi(\infty, t_0) \cdot ( \sigma_c / E_c )",
            numeric_equation=rf"{self.phi_inf_t0:.{n}f} \cdot ( {self.sigma_c:.{n}f} / {self.e_c:.{n}f} )",
            comparison_operator_label="=",
        )



"""Testing formula 6.43 of EN 1992-1-1:2004."""

import pytest

from blueprints.codes.eurocode.en_1992_1_1_2004.chapter_6_ultimate_limit_state.formula_6_43 import Form6Dot43BetaRectangular
from blueprints.validations import LessOrEqualToZeroError, NegativeValueError


class TestForm6Dot43BetaRectangular:
    """Validation for formula 6.43 from EN 1992-1-1:2004."""

    def test_evaluation(self) -> None:
        """Tests the evaluation of the result."""
        # Example values
        ey = 300.0
        ez = 200.0
        by = 400.0
        bz = 500.0

        # Object to test
        formula = Form6Dot43BetaRectangular(ey=ey, ez=ez, by=by, bz=bz)

        # Expected result, manually calculated
        manually_calculated_result = 2.405844942

        assert formula == pytest.approx(expected=manually_calculated_result, rel=1e-4)

    @pytest.mark.parametrize(
        ("ey", "ez", "by", "bz"),
        [
            (-300.0, 200.0, 400.0, 500.0),  # ey is negative
            (300.0, -200.0, 400.0, 500.0),  # ez is negative
            (300.0, 200.0, -400.0, 500.0),  # by is negative
            (300.0, 200.0, 400.0, -500.0),  # bz is negative
            (300.0, 200.0, 0.0, 500.0),  # by is zero
            (300.0, 200.0, 400.0, 0.0),  # bz is zero
        ],
    )
    def test_raise_error_when_invalid_values_are_given(self, ey: float, ez: float, by: float, bz: float) -> None:
        """Test invalid values."""
        with pytest.raises((NegativeValueError, LessOrEqualToZeroError)):
            Form6Dot43BetaRectangular(ey=ey, ez=ez, by=by, bz=bz)

    @pytest.mark.parametrize(
        ("representation", "expected"),
        [
            (
                "complete",
                r"\beta = 1 + 1.8 \cdot \sqrt{\left(\frac{e_y}{b_z}\right)^2 + \left(\frac{e_z}{b_y}\right)^2} "
                r"= 1 + 1.8 \cdot \sqrt{\left(\frac{300.000}{500.000}\right)^2 + \left(\frac{200.000}{400.000}\right)^2} = 2.406 \ -",
            ),
            ("short", r"\beta = 2.406 \ -"),
        ],
    )
    def test_latex(self, representation: str, expected: str) -> None:
        """Test the latex representation of the formula."""
        # Example values
        ey = 300.0
        ez = 200.0
        by = 400.0
        bz = 500.0

        # Object to test
        latex = Form6Dot43BetaRectangular(ey=ey, ez=ez, by=by, bz=bz).latex()

        actual = {
            "complete": latex.complete,
            "short": latex.short,
        }

        assert expected == actual[representation], f"{representation} representation failed."



"""Testing formula 6.29 of NEN-EN 1993-1-1+A1:2016."""

import pytest

from blueprints.codes.eurocode.en_1993_1_1_2005.chapter_6_ultimate_limit_state.formula_6_29 import Form6Dot29ReducedYieldStrength
from blueprints.validations import LessOrEqualToZeroError, NegativeValueError


class TestForm6Dot29ReducedYieldStrength:
    """Validation for formula 6.29 from NEN-EN 1993-1-1+A1:2016."""

    def test_evaluation(self) -> None:
        """Tests the evaluation of the result."""
        # Example values
        rho = 0.75
        f_y = 355.0  # MPa

        # Object to test
        formula = Form6Dot29ReducedYieldStrength(rho=rho, f_y=f_y)

        # Expected result, manually calculated
        manually_calculated_result = 88.75  # MPa

        assert formula == pytest.approx(expected=manually_calculated_result, rel=1e-4)

    @pytest.mark.parametrize(
        ("rho", "f_y"),
        [
            (-0.75, 355.0),  # rho is negative
            (0.75, -355.0),  # f_y is negative
            (1.75, 355.0),  # one_minus_rho is negative
        ],
    )
    def test_raise_error_when_invalid_values_are_given(self, rho: float, f_y: float) -> None:
        """Test invalid values."""
        with pytest.raises((NegativeValueError, LessOrEqualToZeroError)):
            Form6Dot29ReducedYieldStrength(rho=rho, f_y=f_y)

    @pytest.mark.parametrize(
        ("representation", "expected"),
        [
            (
                "complete",
                r"f_{y,red} = (1 - \rho) \cdot f_y = (1 - 0.750) \cdot 355.000 = 88.750 \ MPa",
            ),
            (
                "complete_with_units",
                r"f_{y,red} = (1 - \rho) \cdot f_y = (1 - 0.750) \cdot 355.000 \ MPa = 88.750 \ MPa",
            ),
            ("short", r"f_{y,red} = 88.750 \ MPa"),
        ],
    )
    def test_latex(self, representation: str, expected: str) -> None:
        """Test the latex representation of the formula."""
        # Example values
        rho = 0.75
        f_y = 355.0  # MPa

        # Object to test
        latex = Form6Dot29ReducedYieldStrength(rho=rho, f_y=f_y).latex()

        actual = {
            "complete": latex.complete,
            "complete_with_units": latex.complete_with_units,
            "short": latex.short,
        }

        assert expected == actual[representation], f"{representation} representation failed."



"""Formula 5.39 from EN 1992-1-1:2004: Chapter 5 - Structural Analysis."""

from blueprints.codes.eurocode.en_1992_1_1_2004 import EN_1992_1_1_2004
from blueprints.codes.formula import Formula
from blueprints.codes.latex_formula import LatexFormula
from blueprints.type_alias import DIMENSIONLESS, KNM
from blueprints.validations import raise_if_less_or_equal_to_zero, raise_if_negative


class Form5Dot39SimplifiedCriterionBiaxialBending(Formula):
    r"""Class representing formula 5.39 for simplified criterion for biaxial bending."""

    label = "5.39"
    source_document = EN_1992_1_1_2004

    def __init__(
        self,
        m_edz: KNM,
        m_rdz: KNM,
        m_edy: KNM,
        m_rdy: KNM,
        a: DIMENSIONLESS,
    ) -> None:
        r"""Simplified criterion for biaxial bending.

        EN 1992-1-1:2004 art.5.8.9(4) - Formula (5.39)

        Parameters
        ----------
        m_edz : KNM
            [$M_{Edz}$] Design moment including second order moment in z-direction [$kNm$].
        m_rdz : KNM
            [$M_{Rdz}$] Design moment resistance in z-direction [$kNm$].
        m_edy : KNM
            [$M_{Edy}$] Design moment including second order moment in y-direction [$kNm$].
        m_rdy : KNM
            [$M_{Rdy}$] Design moment resistance in y-direction [$kNm$].
        a : DIMENSIONLESS
            [$a$] Exponent for the interaction formula, for circular and elliptical cross sections: a = 2, for rectangular see table [-].
        """
        super().__init__()
        self.m_edz = m_edz
        self.m_rdz = m_rdz
        self.m_edy = m_edy
        self.m_rdy = m_rdy
        self.a = a

    @staticmethod
    def _evaluate(
        m_edz: KNM,
        m_rdz: KNM,
        m_edy: KNM,
        m_rdy: KNM,
        a: DIMENSIONLESS,
    ) -> bool:
        """Evaluates the formula, for more information see the __init__ method."""
        raise_if_negative(m_edz=m_edz, m_edy=m_edy)
        raise_if_less_or_equal_to_zero(m_rdz=m_rdz, m_rdy=m_rdy, a=a)

        return (m_edz / m_rdz) ** a + (m_edy / m_rdy) ** a <= 1

    def latex(self, n: int = 3) -> LatexFormula:
        """Returns LatexFormula object for formula 5.39."""
        return LatexFormula(
            return_symbol=r"CHECK",
            result="OK" if self.__bool__() else "\\text{Not OK}",
            equation=r"\left( \frac{M_{Edz}}{M_{Rdz}} \right)^{a} + \left( \frac{M_{Edy}}{M_{Rdy}} \right)^{a} \leq 1",
            numeric_equation=rf"\left( \frac{{{self.m_edz:.{n}f}}}{{{self.m_rdz:.{n}f}}} \right)^{{{self.a:.{n}f}}} + "
            rf"\left( \frac{{{self.m_edy:.{n}f}}}{{{self.m_rdy:.{n}f}}} \right)^{{{self.a:.{n}f}}} \leq 1",
            comparison_operator_label="\\to",
            unit="",
        )



"""Test suite for ISteelProfile."""

from unittest.mock import MagicMock

import pytest
from matplotlib import pyplot as plt
from matplotlib.figure import Figure

from blueprints.codes.eurocode.en_1993_1_1_2005.chapter_3_materials.table_3_1 import SteelStrengthClass
from blueprints.materials.steel import SteelMaterial
from blueprints.structural_sections.steel.steel_cross_sections.i_profile import ISteelProfile
from blueprints.structural_sections.steel.steel_cross_sections.standard_profiles.heb import HEB


class TestISteelProfile:
    """Test suite for ISteelProfile."""

    def test_alias(self, i_profile: ISteelProfile) -> None:
        """Test the alias of the I-profile."""
        expected_alias = "HEB360"
        assert i_profile.name == expected_alias

    def test_steel_volume_per_meter(self, i_profile: ISteelProfile) -> None:
        """Test the steel volume per meter."""
        expected_volume = 1.806e-2  # m³/m
        assert pytest.approx(i_profile.volume_per_meter, rel=1e-2) == expected_volume

    def test_steel_weight_per_meter(self, i_profile: ISteelProfile) -> None:
        """Test the steel weight per meter."""
        expected_weight = 1.806e-2 * 7850  # kg/m
        assert pytest.approx(i_profile.weight_per_meter, rel=1e-2) == expected_weight

    def test_steel_area(self, i_profile: ISteelProfile) -> None:
        """Test the steel cross-sectional area."""
        expected_area = 1.806e4  # mm²
        assert pytest.approx(i_profile.area, rel=1e-2) == expected_area

    @pytest.mark.slow
    def test_plot(self, i_profile: ISteelProfile) -> None:
        """Test the plot method (ensure it runs without errors)."""
        fig: Figure = i_profile.plot()
        assert isinstance(fig, plt.Figure)

    def test_plot_mocked(self, i_profile: ISteelProfile, mock_section_properties: MagicMock) -> None:  # noqa: ARG002
        """Test the plotting of the I-profile shapes with mocked section properties."""
        fig: Figure = i_profile.plot()
        assert isinstance(fig, plt.Figure)

    def test_geometry(self, i_profile: ISteelProfile) -> None:
        """Test the geometry of the I profile."""
        expected_geometry = i_profile.geometry
        assert expected_geometry is not None

    def test_get_profile_with_corrosion(self) -> None:
        """Test the EHB profile with 20 mm corrosion applied."""
        # Ensure the profile raises an error if fully corroded
        with pytest.raises(ValueError, match="The profile has fully corroded."):
            ISteelProfile.from_standard_profile(
                profile=HEB.HEB360,
                steel_material=SteelMaterial(SteelStrengthClass.S355),
                corrosion=20,  # mm
            )

    def test_corrosion_in_name(self) -> None:
        """Test that the name includes corrosion information."""
        i_profile_with_corrosion = ISteelProfile.from_standard_profile(
            profile=HEB.HEB360,
            steel_material=SteelMaterial(SteelStrengthClass.S355),
            corrosion=2,  # mm
        )
        expected_name_with_corrosion = "HEB360 (corrosion: 2 mm)"
        assert i_profile_with_corrosion.name == expected_name_with_corrosion



"""Testing formula 5.2 of EN 1993-5:2007."""

import pytest

from blueprints.codes.eurocode.en_1993_5_2007.chapter_5_ultimate_limit_states.formula_5_2 import Form5Dot2DesignMomentResistanceClass1Or2
from blueprints.validations import LessOrEqualToZeroError


class TestForm5Dot2DesignMomentResistanceClass1Or2:
    """Validation for formula 5.2 from EN 1993-5:2007."""

    def test_evaluation(self) -> None:
        """Test the evaluation of the result."""
        # Example values
        beta_b = 0.5  # Dimensionless
        w_pl = 20  # MM3
        f_y = 200  # MPA
        gamma_m_0 = 0.8  # Dimensionless
        form = Form5Dot2DesignMomentResistanceClass1Or2(beta_b=beta_b, w_pl=w_pl, f_y=f_y, gamma_m_0=gamma_m_0)

        # Expected result, manually calculated
        expected = 0.0025  # KNM

        assert form == pytest.approx(expected)

    @pytest.mark.parametrize(
        ("beta_b", "w_pl", "f_y", "gamma_m_0"),
        [
            (-0.5, 1, 1, 1),  # beta_b is negative
            (0, 1, 1, 1),  # beta_b is zero
            (1, -0.5, 1, 1),  # w_pl is negative
            (1, 0, 1, 1),  # w_pl is zero
            (1, 1, -0.5, 1),  # f_y is negative
            (1, 1, 0, 1),  # f_y is zero
            (1, 1, 1, -0.5),  # gamma_m_0 is negative
            (1, 1, 1, 0),  # gamma_m_0 is zero
        ],
    )
    def test_raise_error_when_negative_or_zero_is_given(self, beta_b: float, w_pl: float, f_y: float, gamma_m_0: float) -> None:
        """Test a negative and zero value for parameters beta_b, w_pl, f_y, gamma_m_0."""
        with pytest.raises(LessOrEqualToZeroError):
            Form5Dot2DesignMomentResistanceClass1Or2(beta_b=beta_b, w_pl=w_pl, f_y=f_y, gamma_m_0=gamma_m_0)

    @pytest.mark.parametrize(
        ("representation", "expected"),
        [
            (
                "complete",
                r"M_{c,Rd} = \beta_B W_{pl} f_y / \gamma_{M0} = 0.5 \cdot 20 \cdot 200 / 0.8 / 1000000 = 0.00",
            ),
            ("short", r"M_{c,Rd} = 0.00"),
            (
                "string",
                r"M_{c,Rd} = \beta_B W_{pl} f_y / \gamma_{M0} = 0.5 \cdot 20 \cdot 200 / 0.8 / 1000000 = 0.00",
            ),
        ],
    )
    def test_latex_output(self, representation: str, expected: str) -> None:
        """Test the latex implementation."""
        beta_b = 0.5  # Dimensionless
        w_pl = 20  # MM3
        f_y = 200  # MPA
        gamma_m_0 = 0.8  # Dimensionless

        form_5_2_latex = Form5Dot2DesignMomentResistanceClass1Or2(
            beta_b=beta_b,
            w_pl=w_pl,
            f_y=f_y,
            gamma_m_0=gamma_m_0,
        ).latex()

        actual = {
            "complete": form_5_2_latex.complete,
            "short": form_5_2_latex.short,
            "string": str(form_5_2_latex),
        }

        assert actual[representation] == expected, f"{representation} representation failed."



"""Testing formula 9.1N of EN 1992-1-1:2004."""

import pytest

from blueprints.codes.eurocode.en_1992_1_1_2004.chapter_9_detailling_and_specific_rules.formula_9_1n import (
    Form9Dot1nMinimumTensileReinforcementBeam,
)
from blueprints.validations import NegativeValueError


class TestForm9Dot1nMinimumTensileReinforcementBeam:
    """Validation for formula 9.1N from EN 1992-1-1:2004."""

    def test_evaluation_first_term_decisive(self) -> None:
        """Test the evaluation of the result, if the first term of the formula is decisive."""
        # Example values
        f_ctm = 2  # MPa
        f_yk = 355  # MPa
        b_t = 50  # mm
        d = 150  # mm
        form_9_1n = Form9Dot1nMinimumTensileReinforcementBeam(f_ctm=f_ctm, f_yk=f_yk, b_t=b_t, d=d)

        # Expected result, manually calculated
        manually_calculated_result = 780 / 71

        assert form_9_1n == pytest.approx(expected=manually_calculated_result, rel=1e-4)

    def test_evaluation_second_term_decisive(self) -> None:
        """Test the evaluation of the result, if the second term of the formula is decisive."""
        # Example values
        f_ctm = 2  # MPa
        f_yk = 540  # MPa
        b_t = 50  # mm
        d = 150  # mm
        form_9_1n = Form9Dot1nMinimumTensileReinforcementBeam(f_ctm=f_ctm, f_yk=f_yk, b_t=b_t, d=d)

        # Expected result, manually calculated
        manually_calculated_result = 39 / 4

        assert form_9_1n == pytest.approx(expected=manually_calculated_result, rel=1e-4)

    def test_raise_error_when_negative_f_ctm_is_given(self) -> None:
        """Test if error is raised when f_ctm is negative."""
        # Example values
        f_ctm = -2  # MPa
        f_yk = 355  # MPa
        b_t = 50  # mm
        d = 150  # mm

        with pytest.raises(NegativeValueError):
            Form9Dot1nMinimumTensileReinforcementBeam(f_ctm=f_ctm, f_yk=f_yk, b_t=b_t, d=d)

    def test_raise_error_when_negative_f_yk_is_given(self) -> None:
        """Test if error is raised when f_yk is negative."""
        # Example values
        f_ctm = 2  # MPa
        f_yk = -355  # MPa
        b_t = 50  # mm
        d = 150  # mm

        with pytest.raises(NegativeValueError):
            Form9Dot1nMinimumTensileReinforcementBeam(f_ctm=f_ctm, f_yk=f_yk, b_t=b_t, d=d)

    def test_raise_error_when_negative_b_t_is_given(self) -> None:
        """Test if error is raised when b_t is negative."""
        # Example values
        f_ctm = 2  # MPa
        f_yk = 355  # MPa
        b_t = -50  # mm
        d = 150  # mm

        with pytest.raises(NegativeValueError):
            Form9Dot1nMinimumTensileReinforcementBeam(f_ctm=f_ctm, f_yk=f_yk, b_t=b_t, d=d)

    def test_raise_error_when_negative_d_is_given(self) -> None:
        """Test if error is raised when d is negative."""
        # Example values
        f_ctm = 2  # MPa
        f_yk = 355  # MPa
        b_t = 50  # mm
        d = -150  # mm

        with pytest.raises(NegativeValueError):
            Form9Dot1nMinimumTensileReinforcementBeam(f_ctm=f_ctm, f_yk=f_yk, b_t=b_t, d=d)

    @pytest.mark.parametrize(
        ("representation", "expected_result"),
        [
            (
                "complete",
                r"A_{s,min} = \max \left\{0.26 \cdot \frac{f_{ctm}}{f_{yk}} \cdot b_t \cdot d; 0.0013 \cdot b_t \cdot d\right\} = "
                r"\max \left\{0.26 \cdot \frac{2.00}{355.00} \cdot 50.00 \cdot 150.00; 0.0013 \cdot 50.00 \cdot 150.00\right\} = 10.99",
            ),
            ("short", "A_{s,min} = 10.99"),
        ],
    )
    def test_latex(self, representation: str, expected_result: str) -> None:
        """Test the latex representation."""
        # Example values
        f_ctm = 2  # MPa
        f_yk = 355  # MPa
        b_t = 50  # mm
        d = 150  # mm

        latex = Form9Dot1nMinimumTensileReinforcementBeam(f_ctm=f_ctm, f_yk=f_yk, b_t=b_t, d=d).latex()

        actual = {
            "complete": latex.complete,
            "short": latex.short,
        }

        assert actual[representation] == expected_result, f"{representation} representation failed."



"""Formula 6.58 and 6.59 from EN 1992-1-1:2004: Chapter 6 - Ultimate limit state."""

from blueprints.codes.eurocode.en_1992_1_1_2004 import EN_1992_1_1_2004
from blueprints.codes.formula import Formula
from blueprints.codes.latex_formula import LatexFormula, latex_replace_symbols
from blueprints.type_alias import MM, N
from blueprints.validations import raise_if_less_or_equal_to_zero, raise_if_negative


class Form6Dot58And59TensileForce(Formula):
    r"""Class representing formula 6.58 and 6.59 for the calculation of the tensile force [$T$]."""

    label = "6.58/6.59"
    source_document = EN_1992_1_1_2004

    def __init__(
        self,
        f: N,
        a: MM,
        b: MM,
        capital_h: MM,
    ) -> None:
        r"""[$T$] Tensile force [$N$].

        EN 1992-1-1:2004 art.6.5.3(3) - Formula (6.58 and 6.59)

        Parameters
        ----------
        f : N
            [$F$] Applied force [$N$].
        a : MM
            [$a$] Width of the concentrated force [$mm$].
        b : MM
            [$b$] Width of the locally widened section [$mm$].
        capital_h : MM
            [$H$] Height of the widened section. Also used to calculate h: $h=H/2$ [$mm$].
        """
        super().__init__()
        self.f = f
        self.a = a
        self.b = b
        self.capital_h = capital_h

    @staticmethod
    def _evaluate(
        f: N,
        a: MM,
        b: MM,
        capital_h: MM,
    ) -> N:
        """Evaluates the formula, for more information see the __init__ method."""
        raise_if_negative(f=f, a=a)
        raise_if_less_or_equal_to_zero(b=b, capital_h=capital_h)

        # calculate h from capital_h with Figure 6.25b
        h = capital_h / 2
        if b <= capital_h / 2:
            return 1 / 4 * (b - a) / b * f
        return 1 / 4 * (1 - 0.7 * a / h) * f

    def latex(self, n: int = 3) -> LatexFormula:
        r"""Returns LatexFormula object for formula 6.58/6.59."""
        _equation: str = (
            r"\begin{cases} \frac{1}{4} \cdot \frac{ b - a}{ b} \cdot F & \text{if } b \leq "
            r"\frac{H}{2} \\ \frac{1}{4} \cdot \left(1 - 0.7 \cdot \frac{ a}{\frac{H}{2}}\right) "
            r"\cdot F & \text{if } b > \frac{H}{2} \end{cases}"
        )
        _numeric_equation: str = latex_replace_symbols(
            _equation,
            {
                "F": f"{self.f:.{n}f}",
                " a": f" {self.a:.{n}f}",
                " b": f" {self.b:.{n}f}",
                "H": f"{self.capital_h:.{n}f}",
            },
            False,
        )
        return LatexFormula(
            return_symbol=r"T",
            result=f"{self:.{n}f}",
            equation=_equation,
            numeric_equation=_numeric_equation,
            comparison_operator_label="=",
            unit="N",
        )



"""Formula 7.3 from EN 1992-1-1:2004: Chapter 7 - Serviceability limit state (SLS)."""

from blueprints.codes.eurocode.en_1992_1_1_2004 import EN_1992_1_1_2004
from blueprints.codes.formula import Formula
from blueprints.codes.latex_formula import LatexFormula
from blueprints.type_alias import DIMENSIONLESS, KN, MM2, MPA
from blueprints.unit_conversion import KN_TO_N


class Form7Dot3CoefficientKc(Formula):
    """Class representing the formula 7.3 for the coefficient kc for flanges of tubular cross-sections and T-sections."""

    label = "7.3"
    source_document = EN_1992_1_1_2004

    def __init__(
        self,
        f_cr: KN,
        a_ct: MM2,
        f_ct_eff: MPA,
    ) -> None:
        r"""[$kc$] Calculates kc for flanges of tubular cross-sections and T-sections [$-$].

        EN 1992-1-1:2004 art.7.3.2(2) - Formula (7.3)

        Parameters
        ----------
        f_cr : KN
            [$F_{cr}$] Absolute value of the tensile force within the flange immediately before cracking due to the cracking moment calculated with
            [$f_{ct,eff}$] [$kN$].
        a_ct : MM2
            [$A_{ct}$] Area of the concrete within the tension zone. The tension zone is that part of the cross-section that,
            according to the calculation, is under tension just before the first crack occurs [$mm^2$].
        f_ct_eff : MPA
            [$f_{ct,eff}$] Average value of the tensile strength of the concrete at the time when the first cracks can be expected [$MPa$].
        """
        super().__init__()
        self.f_cr = f_cr
        self.a_ct = a_ct
        self.f_ct_eff = f_ct_eff

    @staticmethod
    def _evaluate(
        f_cr: KN,
        a_ct: MM2,
        f_ct_eff: MPA,
    ) -> DIMENSIONLESS:
        """Evaluates the formula, for more information see the __init__ method."""
        if a_ct <= 0:
            raise ValueError("The value of a_ct must be greater than zero.")
        if f_ct_eff <= 0:
            raise ValueError("The value of f_ct_eff must be greater than zero.")
        return max(0.9 * (abs(f_cr) * KN_TO_N / (a_ct * f_ct_eff)), 0.5)

    def latex(self, n: int = 3) -> LatexFormula:
        """Returns LatexFormula object for formula 7.3."""
        return LatexFormula(
            return_symbol=r"k_c",
            result=f"{self:.{n}f}",
            equation=r"\max\left(0.9 \cdot \frac{F_{cr}}{A_{ct} \cdot f_{ct,eff}}, 0.5\right)",
            numeric_equation=rf"\max\left(0.9 \cdot \frac{{{self.f_cr:.{n}f}}}{{{self.a_ct:.{n}f} \cdot {self.f_ct_eff:.{n}f}}}, 0.5\right)",
            comparison_operator_label="=",
            unit="-",
        )



# mypy: disable-error-code="operator"
"""Tests for the Exposure classes."""

from functools import total_ordering

import pytest

from blueprints.codes.eurocode.en_1992_1_1_2004.chapter_4_durability_and_cover._base_classes.exposure_classes import (
    Exposure,
    ExposureClassesBase,
)


@total_ordering
class DummyExposureSubclass(Exposure):
    """Dummy Exposure subclass for testing purposes."""

    NA = "Not applicable"
    DUMMY1 = "DUMMY1"
    DUMMY2 = "DUMMY2"
    DUMMY3 = "DUMMY3"

    @staticmethod
    def exposure_class_description() -> str:
        """Return the description of the DummyExposureSubclass."""
        return "Dummy exposure subclass"

    def description_of_the_environment(self) -> str:
        """Return the description of the environment."""
        return "Dummy environment"


@total_ordering
class DummyCarbonation(Exposure):
    """Dummy Carbonation subclass for testing purposes."""

    NA = "Not applicable"
    XC1 = "XC1"

    @staticmethod
    def exposure_class_description() -> str:
        """Return the description of the DummyCarbonation."""
        return "Dummy carbonation subclass"

    def description_of_the_environment(self) -> str:
        """Return the description of the environment."""
        return "Dummy environment"


@total_ordering
class DummyChloride(Exposure):
    """Dummy Chloride subclass for testing purposes."""

    NA = "Not applicable"
    XD1 = "XD1"

    @staticmethod
    def exposure_class_description() -> str:
        """Return the description of the DummyChloride."""
        return "Dummy chloride subclass"

    def description_of_the_environment(self) -> str:
        """Return the description of the environment."""
        return "Dummy environment"


@total_ordering
class DummyChlorideSeawater(Exposure):
    """Dummy ChlorideSeawater subclass for testing purposes."""

    NA = "Not applicable"
    XS1 = "XS1"

    @staticmethod
    def exposure_class_description() -> str:
        """Return the description of the DummyChlorideSeawater."""
        return "Dummy chloride seawater subclass"

    def description_of_the_environment(self) -> str:
        """Return the description of the environment."""
        return "Dummy environment"


@total_ordering
class DummyFreezeThaw(Exposure):
    """Dummy FreezeThaw subclass for testing purposes."""

    NA = "Not applicable"
    XF1 = "XF1"

    @staticmethod
    def exposure_class_description() -> str:
        """Return the description of the DummyFreezeThaw."""
        return "Dummy freeze thaw subclass"

    def description_of_the_environment(self) -> str:
        """Return the description of the environment."""
        return "Dummy environment"


@total_ordering
class DummyChemical(Exposure):
    """Dummy Chemical subclass for testing purposes."""

    NA = "Not applicable"
    XA1 = "XA1"

    @staticmethod
    def exposure_class_description() -> str:
        """Return the description of the DummyChemical."""
        return "Dummy chemical subclass"

    def description_of_the_environment(self) -> str:
        """Return the description of the environment."""
        return "Dummy environment"


class DummyExposureClasses(ExposureClassesBase):
    """Dummy ExposureClassesBase subclass for testing purposes."""

    def __init__(
        self,
        dummy_carbonation: DummyCarbonation,
        dummy_chloride: DummyChloride,
        dummy_chloride_seawater: DummyChlorideSeawater,
        dummy_freeze_thaw: DummyFreezeThaw,
        dummy_chemical: DummyChemical,
    ) -> None:
        """Initialize the DummyExposureClassesBase with dummy exposure classes."""
        self.dummy_carbonation = dummy_carbonation
        self.dummy_chloride = dummy_chloride
        self.dummy_chloride_seawater = dummy_chloride_seawater
        self.dummy_freeze_thaw = dummy_freeze_thaw
        self.dummy_chemical = dummy_chemical


class TestExposure:
    """Testing Exposure parent class."""

    def test_initiate_abc_class(self) -> None:
        """Check if initiating the Exposure class raises a TypeError."""
        with pytest.raises(TypeError):
            _ = Exposure()  # type: ignore[abstract, call-arg]

    def test_equal_to_operator(self) -> None:
        """Check if the == operator is working correctly."""
        assert DummyExposureSubclass.DUMMY1 == DummyExposureSubclass.DUMMY1
        assert DummyExposureSubclass.DUMMY2 == DummyExposureSubclass.DUMMY2
        assert DummyExposureSubclass.DUMMY3 == DummyExposureSubclass.DUMMY3

    def test_not_equal_to_operator(self) -> None:
        """Check if the != operator is working correctly."""
        assert DummyExposureSubclass.DUMMY1 != DummyExposureSubclass.DUMMY2
        assert DummyExposureSubclass.DUMMY2 != DummyExposureSubclass.DUMMY3
        assert DummyExposureSubclass.DUMMY3 != DummyExposureSubclass.DUMMY1

    def test_greather_than_operator(self) -> None:
        """Check if the > operator is working correctly."""
        assert DummyExposureSubclass.DUMMY3 > DummyExposureSubclass.DUMMY2
        assert DummyExposureSubclass.DUMMY2 > DummyExposureSubclass.DUMMY1
        assert DummyExposureSubclass.DUMMY1 > DummyExposureSubclass.NA

    def test_greather_than_equal_to_operator(self) -> None:
        """Check if the >= operator is working correctly."""
        # Testing greater than
        assert DummyExposureSubclass.DUMMY3 >= DummyExposureSubclass.DUMMY2
        assert DummyExposureSubclass.DUMMY2 >= DummyExposureSubclass.DUMMY1
        assert DummyExposureSubclass.DUMMY1 >= DummyExposureSubclass.NA
        # Testing equal to
        assert DummyExposureSubclass.DUMMY3 >= DummyExposureSubclass.DUMMY3
        assert DummyExposureSubclass.DUMMY1 >= DummyExposureSubclass.DUMMY1

    def test_lesser_than_operator(self) -> None:
        """Check if the < operator is working correctly."""
        assert DummyExposureSubclass.DUMMY1 < DummyExposureSubclass.DUMMY2
        assert DummyExposureSubclass.DUMMY2 < DummyExposureSubclass.DUMMY3
        assert DummyExposureSubclass.NA < DummyExposureSubclass.DUMMY1

    def test_lesser_than_equal_to_operator(self) -> None:
        """Check if the <= operator is working correctly."""
        # Testing lesser than
        assert DummyExposureSubclass.DUMMY1 <= DummyExposureSubclass.DUMMY2
        assert DummyExposureSubclass.DUMMY2 <= DummyExposureSubclass.DUMMY3
        assert DummyExposureSubclass.NA <= DummyExposureSubclass.DUMMY1
        # Testing equal to
        assert DummyExposureSubclass.DUMMY1 <= DummyExposureSubclass.DUMMY1
        assert DummyExposureSubclass.DUMMY3 <= DummyExposureSubclass.DUMMY3

    def test_options(self) -> None:
        """Check if the options method returns all the possible options within an exposure class."""
        assert DummyExposureSubclass.options() == ["Not applicable", "DUMMY1", "DUMMY2", "DUMMY3"]

    def test_exposure_class_description_implemented(self) -> None:
        """Check if the exposure_class_description method returns the description of the subclass."""
        assert DummyExposureSubclass.exposure_class_description() == "Dummy exposure subclass"

    def test_description_of_the_environment(self) -> None:
        """Check if the description_of_the_environment method returns the description of the environment."""
        assert DummyExposureSubclass.DUMMY1.description_of_the_environment() == "Dummy environment"

    def test_notation(self) -> None:
        """Check if the notation method returns the correct notation."""
        assert DummyExposureSubclass.notation() == "DUMMY"
        assert DummyExposureSubclass.DUMMY1.notation() == "DUMMY"
        assert DummyExposureSubclass.DUMMY2.notation() == "DUMMY"
        assert DummyExposureSubclass.DUMMY3.notation() == "DUMMY"

    def test_notation_raises_error(self) -> None:
        """Check if the notation method raises an error when called on the base class."""

        class BadNotation(Exposure):
            """Dummy Exposure subclass with bad notation."""

            NA = "Not applicable"
            DUMMY1 = "dummy1"
            DUMMY2 = "DUMMY"
            DUMMY3 = "Dummy3"

            @staticmethod
            def exposure_class_description() -> str:
                """Dummy implementation of abstract method."""
                return ""

            def description_of_the_environment(self) -> str:
                """Dummy implementation of abstract method."""
                return ""

        with pytest.raises(ValueError):
            _ = BadNotation.notation()


class TestCarbonation:
    """Testing Carbonation class."""

    def test_exposure_class_description_implemented(self) -> None:
        """Check if the exposure_class_description method returns the description of the subclass."""
        assert DummyCarbonation.exposure_class_description() == "Dummy carbonation subclass"

    def test_description_of_the_environment(self) -> None:
        """Check if the description_of_the_environment method returns the description of the environment."""
        assert DummyCarbonation.XC1.description_of_the_environment() == "Dummy environment"

    def test_notation(self) -> None:
        """Check if the notation method returns the correct notation."""
        assert DummyCarbonation.notation() == "XC"


class TestChloride:
    """Testing Chloride class."""

    def test_exposure_class_description_implemented(self) -> None:
        """Check if the exposure_class_description method returns the description of the subclass."""
        assert DummyChloride.exposure_class_description() == "Dummy chloride subclass"

    def test_description_of_the_environment(self) -> None:
        """Check if the description_of_the_environment method returns the description of the environment."""
        assert DummyChloride.XD1.description_of_the_environment() == "Dummy environment"

    def test_notation(self) -> None:
        """Check if the notation method returns the correct notation."""
        assert DummyChloride.notation() == "XD"


class TestChlorideSeawater:
    """Testing ChlorideSeawater class."""

    def test_exposure_class_description_implemented(self) -> None:
        """Check if the exposure_class_description method returns the description of the subclass."""
        assert DummyChlorideSeawater.exposure_class_description() == "Dummy chloride seawater subclass"

    def test_description_of_the_environment(self) -> None:
        """Check if the description_of_the_environment method returns the description of the environment."""
        assert DummyChlorideSeawater.XS1.description_of_the_environment() == "Dummy environment"

    def test_notation(self) -> None:
        """Check if the notation method returns the correct notation."""
        assert DummyChlorideSeawater.notation() == "XS"


class TestFreezeThaw:
    """Testing FreezeThaw class."""

    def test_exposure_class_description_implemented(self) -> None:
        """Check if the exposure_class_description method returns the description of the subclass."""
        assert DummyFreezeThaw.exposure_class_description() == "Dummy freeze thaw subclass"

    def test_description_of_the_environment(self) -> None:
        """Check if the description_of_the_environment method returns the description of the environment."""
        assert DummyFreezeThaw.XF1.description_of_the_environment() == "Dummy environment"

    def test_notation(self) -> None:
        """Check if the notation method returns the correct notation."""
        assert DummyFreezeThaw.notation() == "XF"


class TestChemical:
    """Testing Chemical class."""

    def test_exposure_class_description_implemented(self) -> None:
        """Check if the exposure_class_description method returns the description of the subclass."""
        assert DummyChemical.exposure_class_description() == "Dummy chemical subclass"

    def test_description_of_the_environment(self) -> None:
        """Check if the description_of_the_environment method returns the description of the environment."""
        assert DummyChemical.XA1.description_of_the_environment() == "Dummy environment"

    def test_notation(self) -> None:
        """Check if the notation method returns the correct notation."""
        assert DummyChemical.notation() == "XA"


class TestExposureClasses:
    """Testing ExposureClasses class."""

    def test_str(self) -> None:
        """Check if the __str__ method returns the correct string representation of the exposure classes."""
        exposure_classes = DummyExposureClasses(
            dummy_carbonation=DummyCarbonation("XC1"),
            dummy_chloride=DummyChloride("XD1"),
            dummy_chloride_seawater=DummyChlorideSeawater("Not applicable"),
            dummy_freeze_thaw=DummyFreezeThaw("XF1"),
            dummy_chemical=DummyChemical("XA1"),
        )
        assert str(exposure_classes) == "XC1, XD1, XF1, XA1"

    def test_str_x0(self) -> None:
        """Check if the __str__ method returns the correct string representation of the exposure classes
        when all exposure classes are not applicable.
        """
        exposureclasses = DummyExposureClasses(
            dummy_carbonation=DummyCarbonation("Not applicable"),
            dummy_chloride=DummyChloride("Not applicable"),
            dummy_chloride_seawater=DummyChlorideSeawater("Not applicable"),
            dummy_freeze_thaw=DummyFreezeThaw("Not applicable"),
            dummy_chemical=DummyChemical("Not applicable"),
        )
        assert str(exposureclasses) == "X0"

    def test_no_risk(self) -> None:
        """Check if the no_risk method returns True if the exposure classes are all not applicable."""
        exposureclasses = DummyExposureClasses(
            dummy_carbonation=DummyCarbonation("Not applicable"),
            dummy_chloride=DummyChloride("Not applicable"),
            dummy_chloride_seawater=DummyChlorideSeawater("Not applicable"),
            dummy_freeze_thaw=DummyFreezeThaw("Not applicable"),
            dummy_chemical=DummyChemical("Not applicable"),
        )
        assert exposureclasses.no_risk is True

    def test_no_risk_false(self) -> None:
        """Check if the no_risk method returns False if at least one exposure class is applicable."""
        exposureclasses = DummyExposureClasses(
            dummy_carbonation=DummyCarbonation("XC1"),
            dummy_chloride=DummyChloride("XD1"),
            dummy_chloride_seawater=DummyChlorideSeawater("Not applicable"),
            dummy_freeze_thaw=DummyFreezeThaw("XF1"),
            dummy_chemical=DummyChemical("XA1"),
        )
        assert exposureclasses.no_risk is False

    def test_iter(self) -> None:
        """Check if the __iter__ method returns an iterator over the exposure classes."""
        exposure_classes = DummyExposureClasses(
            dummy_carbonation=DummyCarbonation("XC1"),
            dummy_chloride=DummyChloride("XD1"),
            dummy_chloride_seawater=DummyChlorideSeawater("XS1"),
            dummy_freeze_thaw=DummyFreezeThaw("XF1"),
            dummy_chemical=DummyChemical("XA1"),
        )
        iterator = iter(exposure_classes)
        assert next(iterator) == DummyCarbonation("XC1")
        assert next(iterator) == DummyChloride("XD1")
        assert next(iterator) == DummyChlorideSeawater("XS1")
        assert next(iterator) == DummyFreezeThaw("XF1")
        assert next(iterator) == DummyChemical("XA1")

    def test_from_exposure_list(self) -> None:
        """Check if the from_exposure_list method creates an instance from a sequence of exposure classes."""
        carbonation = "XC1"
        chloride = "XD1"
        chloride_seawater = "XS1"
        freeze_thaw = "XF1"
        exposure_classes = DummyExposureClasses.from_exposure_list([carbonation, chloride, chloride_seawater, freeze_thaw])
        assert exposure_classes.dummy_carbonation == DummyCarbonation(carbonation)
        assert exposure_classes.dummy_chloride == DummyChloride(chloride)
        assert exposure_classes.dummy_chloride_seawater == DummyChlorideSeawater(chloride_seawater)
        assert exposure_classes.dummy_freeze_thaw == DummyFreezeThaw(freeze_thaw)
        assert exposure_classes.dummy_chemical == DummyChemical("Not applicable")

    def test_from_exposure_list_empty(self) -> None:
        """Check if the from_exposure_list method creates an instance with all exposure classes as not applicable."""
        exposure_classes = DummyExposureClasses.from_exposure_list([])
        assert exposure_classes.dummy_carbonation == DummyCarbonation("Not applicable")
        assert exposure_classes.dummy_chloride == DummyChloride("Not applicable")
        assert exposure_classes.dummy_chloride_seawater == DummyChlorideSeawater("Not applicable")
        assert exposure_classes.dummy_freeze_thaw == DummyFreezeThaw("Not applicable")
        assert exposure_classes.dummy_chemical == DummyChemical("Not applicable")


def test_comparing_different_types_raises_error() -> None:
    """Check if comparing different exposure class types, raises TypeError."""
    with pytest.raises(TypeError):
        DummyCarbonation.XC1 > DummyChloride.NA
    with pytest.raises(TypeError):
        DummyChloride.NA == DummyChemical.NA
    with pytest.raises(TypeError):
        DummyFreezeThaw.XF1 <= DummyChlorideSeawater.XS1
    with pytest.raises(TypeError):
        DummyChlorideSeawater.XS1 >= DummyChloride.XD1
    with pytest.raises(TypeError):
        DummyChemical.XA1 < DummyCarbonation.XC1
    with pytest.raises(TypeError):
        DummyChemical.NA != DummyFreezeThaw.XF1



"""Testing formula 6.26 of EN 1993-1-1:2005."""

import pytest

from blueprints.codes.eurocode.en_1993_1_1_2005.chapter_6_ultimate_limit_state.formula_6_26 import Form6Dot26VplTRdIOrHSection
from blueprints.validations import LessOrEqualToZeroError, NegativeValueError


class TestForm6Dot26VplTRdIOrHSection:
    """Validation for formula 6.26 from EN 1993-1-1:2005."""

    def test_evaluation(self) -> None:
        """Tests the evaluation of the result."""
        # Example values
        tau_t_ed = 50.0
        f_y = 355.0
        gamma_m0 = 1.0
        v_pl_rd = 100000.0

        # Object to test
        formula = Form6Dot26VplTRdIOrHSection(tau_t_ed=tau_t_ed, f_y=f_y, gamma_m0=gamma_m0, v_pl_rd=v_pl_rd)

        # Expected result, manually calculated
        manually_calculated_result = 89712.8388597  # N

        assert formula == pytest.approx(manually_calculated_result, rel=1e-4)

    @pytest.mark.parametrize(
        ("tau_t_ed", "v_pl_rd"),
        [
            (-50.0, 100000.0),  # tau_t_ed is negative
            (50.0, -100000.0),  # v_pl_rd is negative
            (1e9, 100000.0),  # square root becomes negative
        ],
    )
    def test_raise_error_when_negative_values_are_given(self, tau_t_ed: float, v_pl_rd: float) -> None:
        """Test invalid values."""
        with pytest.raises(NegativeValueError):
            Form6Dot26VplTRdIOrHSection(tau_t_ed=tau_t_ed, f_y=355.0, gamma_m0=1.0, v_pl_rd=v_pl_rd)

    @pytest.mark.parametrize(
        ("gamma_m0", "f_y"),
        [
            (0.0, 355.0),  # gamma_m0 is zero
            (-1.0, 355.0),  # gamma_m0 is negative
            (1.0, 0.0),  # f_y is zero
            (1.0, -355.0),  # f_y is negative
        ],
    )
    def test_raise_error_when_less_or_equal_to_zero_values_are_given(self, gamma_m0: float, f_y: float) -> None:
        """Test invalid values."""
        with pytest.raises(LessOrEqualToZeroError):
            Form6Dot26VplTRdIOrHSection(tau_t_ed=10.0, f_y=f_y, gamma_m0=gamma_m0, v_pl_rd=100000.0)

    @pytest.mark.parametrize(
        ("representation", "expected"),
        [
            (
                "complete",
                r"V_{pl,T,Rd} = \sqrt{1 - \frac{\tau_{t,Ed}}{1.25 \cdot \left( f_y / \sqrt{3} \right) / \gamma_{M0}}} \cdot V_{pl,Rd} = "
                r"\sqrt{1 - \frac{50.000}{1.25 \cdot \left( 355.000 / \sqrt{3} \right) / 1.000}} \cdot 100000.000 = 89712.839 \ N",
            ),
            ("short", r"V_{pl,T,Rd} = 89712.839 \ N"),
        ],
    )
    def test_latex(self, representation: str, expected: str) -> None:
        """Test the latex representation of the formula."""
        # Example values
        tau_t_ed = 50.0
        f_y = 355.0
        gamma_m0 = 1.0
        v_pl_rd = 100000.0

        # Object to test
        formula = Form6Dot26VplTRdIOrHSection(tau_t_ed=tau_t_ed, f_y=f_y, gamma_m0=gamma_m0, v_pl_rd=v_pl_rd)
        latex = formula.latex()

        actual = {
            "complete": latex.complete,
            "short": latex.short,
        }

        assert expected == actual[representation], f"{representation} representation failed."



"""Tests for constants classes for the calculation of nominal concrete cover."""

from dataclasses import dataclass, field

import pytest

from blueprints.codes.eurocode.en_1992_1_1_2004.chapter_4_durability_and_cover._base_classes.nominal_cover_constants import (
    AbrasionClass,
    CastingSurface,
    NominalConcreteCoverConstantsBase,
)
from blueprints.type_alias import MM


@dataclass(frozen=True)
class DummyConstants(NominalConcreteCoverConstantsBase):
    """Dummy constants for testing purposes."""

    CODE_PREFIX: str = "DUMMY_PREFIX-"
    CODE_SUFFIX: str = ":DUMMY_SUFFIX"
    COVER_INCREASE_FOR_ABRASION_CLASS: dict[AbrasionClass, MM] = field(default_factory=dict)
    COVER_INCREASE_FOR_UNEVEN_SURFACE: MM = 5
    DEFAULT_DELTA_C_DEV: MM = 5

    def __post_init__(self) -> None:
        """Post-initialization method to set default values."""
        if not self.COVER_INCREASE_FOR_ABRASION_CLASS:
            abrasion_class_cover_increase = {
                "NA": 0,
                "XM1": 5,
                "XM2": 10,
                "XM3": 15,
            }
            object.__setattr__(self, "COVER_INCREASE_FOR_ABRASION_CLASS", abrasion_class_cover_increase)

    @staticmethod
    def minimum_cover_with_regard_to_casting_surface(c_min_dur, casting_surface) -> MM:  # noqa: ANN001, ARG004
        """Dummy method for testing purposes."""
        return 0

    @staticmethod
    def minimum_cover_with_regard_to_casting_surface_latex(casting_surface) -> str:  # noqa: ANN001, ARG004
        """Dummy method for testing purposes."""
        return ""


class TestNominalConcreteCoverConstantsBase:
    """Tests for the base class for constants for the calculation of nominal concrete cover."""

    def test_abstract_method(self) -> None:
        """Test that the abstract method raises a TypeError."""
        with pytest.raises(TypeError):
            _ = NominalConcreteCoverConstantsBase("_", "_", 0, {}, 0)  # type: ignore[abstract]

    def test_instantiation(self) -> None:
        """Test that the class can be instantiated."""
        constants = DummyConstants()
        assert constants.COVER_INCREASE_FOR_UNEVEN_SURFACE == 5
        assert constants.COVER_INCREASE_FOR_ABRASION_CLASS == {
            "NA": 0,
            "XM1": 5,
            "XM2": 10,
            "XM3": 15,
        }
        assert constants.DEFAULT_DELTA_C_DEV == 5
        assert constants.CODE_PREFIX == "DUMMY_PREFIX-"
        assert constants.CODE_SUFFIX == ":DUMMY_SUFFIX"
        assert constants.minimum_cover_with_regard_to_casting_surface(0, CastingSurface.PERMANENTLY_EXPOSED) == 0
        assert constants.minimum_cover_with_regard_to_casting_surface_latex(CastingSurface.PERMANENTLY_EXPOSED) == ""



"""Constants for the calculation of nominal concrete cover according to EN 1992-1-1:2004."""

from dataclasses import dataclass, field

from blueprints.codes.eurocode.en_1992_1_1_2004.chapter_4_durability_and_cover._base_classes.nominal_cover_constants import (
    AbrasionClass,
    CastingSurface,
    NominalConcreteCoverConstantsBase,
)
from blueprints.type_alias import MM


@dataclass(frozen=True)
class NominalConcreteCoverConstants(NominalConcreteCoverConstantsBase):
    """Constants for the calculation of nominal concrete cover according to EN 1992-1-1:2004."""

    CODE_PREFIX: str = field(default="", init=False)

    CODE_SUFFIX: str = field(default=":2004", init=False)

    # According to art. 4.4.1.2 (11) from EN 1992-1-1:2004
    COVER_INCREASE_FOR_UNEVEN_SURFACE: MM = field(default=5, init=False)

    # According to art. 4.4.1.3 (1) from EN 1992-1-1:2004
    DEFAULT_DELTA_C_DEV: MM = field(default_factory=int)

    # According to art. 4.4.1.2 (13) from EN 1992-1-1:2004
    COVER_INCREASE_FOR_ABRASION_CLASS: dict[AbrasionClass, MM] = field(default_factory=dict)

    def __post_init__(self) -> None:
        """Post-initialization method to set default values."""
        if not self.COVER_INCREASE_FOR_ABRASION_CLASS:
            abrasion_class_cover_increase = {
                AbrasionClass.NA: 0,
                AbrasionClass.XM1: 5,  # k1 according to art. 4.4.1.2 (13)
                AbrasionClass.XM2: 10,  # k2 according to art. 4.4.1.2 (13)
                AbrasionClass.XM3: 15,  # k3 according to art. 4.4.1.2 (13)
            }
            object.__setattr__(self, "COVER_INCREASE_FOR_ABRASION_CLASS", abrasion_class_cover_increase)
        if not self.DEFAULT_DELTA_C_DEV:
            object.__setattr__(self, "DEFAULT_DELTA_C_DEV", 10)

    @staticmethod
    def minimum_cover_with_regard_to_casting_surface(c_min_dur: MM, casting_surface: CastingSurface) -> MM:
        """Calculate the minimum cover with regard to casting surface according to art. 4.4.1.3 (4) from EN 1992-1-1:2004."""
        match casting_surface:
            case CastingSurface.PERMANENTLY_EXPOSED | CastingSurface.FORMWORK:
                return 0  # No additional requirements
            case CastingSurface.PREPARED_GROUND:
                return c_min_dur + 40  # k1 ≥ c_min,dur + 40
            case CastingSurface.DIRECTLY_AGAINST_SOIL:
                return c_min_dur + 75  # k2 ≥ c_min,dur + 75

    @staticmethod
    def minimum_cover_with_regard_to_casting_surface_latex(casting_surface: CastingSurface) -> str:
        """LateX representation of minimum cover with regard to casting surface according to art. 4.4.1.3 (4) from EN 1992-1-1:2004."""
        match casting_surface:
            case CastingSurface.PERMANENTLY_EXPOSED | CastingSurface.FORMWORK:
                return f"0 (No additional requirements for {casting_surface.value})"
            case CastingSurface.PREPARED_GROUND:
                return f"k1 \\ge c_{{min,dur}} + 40 mm for {casting_surface.value}"
            case CastingSurface.DIRECTLY_AGAINST_SOIL:
                return f"k2 \\ge c_{{min,dur}} + 75 mm for {casting_surface.value}"



"""Unit tests for the HEM enum."""

from blueprints.structural_sections.steel.steel_cross_sections.standard_profiles.hem import HEM


class TestHEM:
    """Tests for the HEM enum."""

    def test_enum_values(self) -> None:
        """Test that enum values are correctly defined."""
        assert HEM.HEM100.value == ("HEM100", 120, 106, 12, 20, 12)
        assert HEM.HEM1000.value == ("HEM1000", 1008, 302, 21, 40, 30)

    def test_enum_membership(self) -> None:
        """Test that specific values are members of the enum."""
        assert "HEM100" in [e.value[0] for e in HEM]
        assert "HEM1000" in [e.value[0] for e in HEM]

    def test_enum_uniqueness(self) -> None:
        """Test that all enum values are unique."""
        values = [e.value for e in HEM]
        assert len(values) == len(set(values))

    def test_enum_attributes(self) -> None:
        """Test that enum attributes are correctly assigned."""
        profile = HEM.HEM100
        assert profile.alias == "HEM100"
        assert profile.total_height == 120
        assert profile.top_flange_width == 106
        assert profile.top_flange_thickness == 20
        assert profile.bottom_flange_width == 106
        assert profile.bottom_flange_thickness == 20
        assert profile.web_thickness == 12
        assert profile.top_radius == 12
        assert profile.bottom_radius == 12



"""Testing formula 6.32 of EN 1992-1-1:2004."""

import pytest

from blueprints.codes.eurocode.en_1992_1_1_2004.chapter_6_ultimate_limit_state.formula_6_32 import Form6Dot32EffectiveDepthSlab
from blueprints.validations import NegativeValueError


class TestForm6Dot32EffectiveDepthSlab:
    """Validation for formula 6.32 from EN 1992-1-1:2004."""

    def test_evaluation(self) -> None:
        """Tests the evaluation of the result."""
        # Example values
        d_y = 200.0
        d_z = 300.0

        # Object to test
        formula = Form6Dot32EffectiveDepthSlab(d_y=d_y, d_z=d_z)

        # Expected result, manually calculated
        manually_calculated_result = 250.0  # mm

        assert formula == pytest.approx(expected=manually_calculated_result, rel=1e-4)

    @pytest.mark.parametrize(
        ("d_y", "d_z"),
        [
            (-200.0, 300.0),  # d_y is negative
            (200.0, -300.0),  # d_z is negative
        ],
    )
    def test_raise_error_when_negative_values_are_given(self, d_y: float, d_z: float) -> None:
        """Test invalid values."""
        with pytest.raises(NegativeValueError):
            Form6Dot32EffectiveDepthSlab(d_y=d_y, d_z=d_z)

    @pytest.mark.parametrize(
        ("representation", "expected"),
        [
            (
                "complete",
                r"d_{eff} = \frac{d_{y} + d_{z}}{2} = \frac{200.000 + 300.000}{2} = 250.000 \ mm",
            ),
            ("short", r"d_{eff} = 250.000 \ mm"),
        ],
    )
    def test_latex(self, representation: str, expected: str) -> None:
        """Test the latex representation of the formula."""
        # Example values
        d_y = 200.0
        d_z = 300.0

        # Object to test
        latex = Form6Dot32EffectiveDepthSlab(d_y=d_y, d_z=d_z).latex()

        actual = {
            "complete": latex.complete,
            "short": latex.short,
        }

        assert expected == actual[representation], f"{representation} representation failed."



"""Testing formula 6.12 of EN 1992-1-1:2004."""

import pytest

from blueprints.codes.eurocode.en_1992_1_1_2004.chapter_6_ultimate_limit_state.formula_6_12 import (
    Form6Dot12CheckMaxEffectiveCrossSectionalAreaShearReinf,
)
from blueprints.validations import LessOrEqualToZeroError, NegativeValueError


class TestForm6Dot12CheckMaxEffectiveCrossSectionalAreaShearReinf:
    """Validation for formula 6.12 from EN 1992-1-1:2004."""

    def test_evaluation(self) -> None:
        """Tests the evaluation of the result."""
        # Example values
        a_sw_max = 100.0  # mm²
        f_ywd = 400.0  # MPa
        b_w = 300.0  # mm
        s = 200.0  # mm
        alpha_cw = 1.0  # dimensionless
        nu_1 = 0.6  # dimensionless
        f_cd = 30.0  # MPa

        # Object to test
        formula = Form6Dot12CheckMaxEffectiveCrossSectionalAreaShearReinf(
            a_sw_max=a_sw_max,
            f_ywd=f_ywd,
            b_w=b_w,
            s=s,
            alpha_cw=alpha_cw,
            nu_1=nu_1,
            f_cd=f_cd,
        )

        # Expected result, manually calculated
        expected_result = True

        assert formula == expected_result

    @pytest.mark.parametrize(
        ("a_sw_max", "f_ywd", "b_w", "s", "alpha_cw", "nu_1", "f_cd"),
        [
            (-100.0, 400.0, 300.0, 200.0, 1.0, 0.6, 30.0),  # a_sw_max is negative
            (100.0, -400.0, 300.0, 200.0, 1.0, 0.6, 30.0),  # f_ywd is negative
            (100.0, 400.0, -300.0, 200.0, 1.0, 0.6, 30.0),  # b_w is negative
            (100.0, 400.0, 300.0, -200.0, 1.0, 0.6, 30.0),  # s is negative
            (100.0, 400.0, 300.0, 200.0, -1.0, 0.6, 30.0),  # alpha_cw is negative
            (100.0, 400.0, 300.0, 200.0, 1.0, -0.6, 30.0),  # nu_1 is negative
            (100.0, 400.0, 300.0, 200.0, 1.0, 0.6, -30.0),  # f_cd is negative
            (100.0, 400.0, 0.0, 200.0, 1.0, 0.6, 30.0),  # b_w is zero
            (100.0, 400.0, 300.0, 0.0, 1.0, 0.6, 30.0),  # s is zero
        ],
    )
    def test_raise_error_when_invalid_values_are_given(
        self, a_sw_max: float, f_ywd: float, b_w: float, s: float, alpha_cw: float, nu_1: float, f_cd: float
    ) -> None:
        """Test invalid values."""
        with pytest.raises((NegativeValueError, LessOrEqualToZeroError)):
            Form6Dot12CheckMaxEffectiveCrossSectionalAreaShearReinf(
                a_sw_max=a_sw_max,
                f_ywd=f_ywd,
                b_w=b_w,
                s=s,
                alpha_cw=alpha_cw,
                nu_1=nu_1,
                f_cd=f_cd,
            )

    @pytest.mark.parametrize(
        ("representation", "expected"),
        [
            (
                "complete",
                r"CHECK \to \frac{A_{sw,max} \cdot f_{ywd}}{b_{w} \cdot s} \leq \frac{1}{2} \cdot \alpha_{cw} \cdot \nu_{1} \cdot f_{cd} \to "
                r"\frac{100.000 \cdot 400.000}{300.000 \cdot 200.000} \leq \frac{1}{2} \cdot 1.000 \cdot 0.600 \cdot 30.000 \to OK",
            ),
            ("short", r"CHECK \to OK"),
        ],
    )
    def test_latex(self, representation: str, expected: str) -> None:
        """Test the latex representation of the formula."""
        # Example values
        a_sw_max = 100.0  # mm²
        f_ywd = 400.0  # MPa
        b_w = 300.0  # mm
        s = 200.0  # mm
        alpha_cw = 1.0  # dimensionless
        nu_1 = 0.6  # dimensionless
        f_cd = 30.0  # MPa

        # Object to test
        latex = Form6Dot12CheckMaxEffectiveCrossSectionalAreaShearReinf(
            a_sw_max=a_sw_max,
            f_ywd=f_ywd,
            b_w=b_w,
            s=s,
            alpha_cw=alpha_cw,
            nu_1=nu_1,
            f_cd=f_cd,
        ).latex()

        actual = {
            "complete": latex.complete,
            "short": latex.short,
        }

        assert expected == actual[representation], f"{representation} representation failed."



"""Testing formula 8.19 of EN 1992-1-1:2004."""

import pytest

from blueprints.codes.eurocode.en_1992_1_1_2004.chapter_8_detailing_of_reinforcement_and_prestressing_tendons.formula_8_19 import (
    Form8Dot19DispersionLength,
)
from blueprints.validations import LessOrEqualToZeroError


class TestForm8Dot19DispersionLength:
    """Validation for formula 8.19 from EN 1992-1-1:2004."""

    def test_evaluation(self) -> None:
        """Test the evaluation of the result."""
        # Example values
        l_pt = 10  # m
        d = 2  # m
        form_8_19 = Form8Dot19DispersionLength(l_pt=l_pt, d=d)

        # Expected result, manually calculated
        manually_calculated_result = 10.198

        assert form_8_19 == pytest.approx(expected=manually_calculated_result, rel=1e-4)

    def test_raise_error_when_negative_l_pt_is_given(self) -> None:
        """Test a negative value for l_pt."""
        # Example values
        l_pt = -10  # m
        d = 2  # m

        with pytest.raises(LessOrEqualToZeroError):
            Form8Dot19DispersionLength(l_pt=l_pt, d=d)

    def test_raise_error_when_negative_d_is_given(self) -> None:
        """Test a negative value for d."""
        # Example values
        l_pt = 10  # m
        d = -2  # m

        with pytest.raises(LessOrEqualToZeroError):
            Form8Dot19DispersionLength(l_pt=l_pt, d=d)

    @pytest.mark.parametrize(
        ("representation", "expected"),
        [
            (
                "complete",
                r"l_{disp} = \sqrt{l_{pt}^2 + d^2} = \sqrt{10.000^2 + 2.000^2} = 10.198",
            ),
            ("short", r"l_{disp} = 10.198"),
        ],
    )
    def test_latex(self, representation: str, expected: str) -> None:
        """Test the latex representation of the formula."""
        # Example values
        l_pt = 10  # m
        d = 2  # m

        # Object to test
        form_8_19_latex = Form8Dot19DispersionLength(l_pt=l_pt, d=d).latex()

        actual = {
            "complete": form_8_19_latex.complete,
            "short": form_8_19_latex.short,
        }

        assert actual[representation] == expected, f"{representation} representation failed."



"""Testing formula 5.7 of EN 1993-5:2007."""

import pytest

from blueprints.codes.eurocode.en_1993_5_2007.chapter_5_ultimate_limit_states.formula_5_7 import Form5Dot7ShearBucklingResistance
from blueprints.validations import LessOrEqualToZeroError


class TestForm5Dot7ShearBucklingResistance:
    """Validation for formula 5.7 from EN 1993-5:2007."""

    def test_evaluation(self) -> None:
        """Test the evaluation of the result."""
        # Example values
        h = 500  # MM
        t_f = 20  # MM
        t_w = 10  # MM
        f_bv = 300  # MPA
        gamma_m_0 = 1.1

        form = Form5Dot7ShearBucklingResistance(
            h=h,
            t_f=t_f,
            t_w=t_w,
            f_bv=f_bv,
            gamma_m_0=gamma_m_0,
        )

        # Expected result, manually calculated
        expected = 1309.09

        assert form == pytest.approx(expected)

    @pytest.mark.parametrize(
        ("h", "t_f", "t_w", "f_bv", "gamma_m_0"),
        [
            (0, 20, 10, 300, 1.1),  # h is zero
            (-500, 20, 10, 300, 1.1),  # h is negative
            (500, -20, 10, 300, 1.1),  # t_f is negative
            (500, 0, 10, 300, 1.1),  # t_f is zero
            (500, 20, -10, 300, 1.1),  # t_w is negative
            (500, 20, 0, 300, 1.1),  # t_w is zero
            (500, 20, 10, 300, 0),  # gamma_m_0 is zero
            (500, 20, 10, 300, -1),  # gamma_m_0 is negative
        ],
    )
    def test_raise_error_when_negative_or_zero_values_are_given(self, h: float, t_f: float, t_w: float, f_bv: float, gamma_m_0: float) -> None:
        """Test a zero and negative value for parameters h, t_f, t_w, f_bv, and gamma_m_0."""
        with pytest.raises(LessOrEqualToZeroError):
            Form5Dot7ShearBucklingResistance(h=h, t_f=t_f, t_w=t_w, f_bv=f_bv, gamma_m_0=gamma_m_0)

    def test_t_f_not_greater_than_h(self) -> None:
        """Test if the thickness of the flange is not greater than the height of the web."""
        with pytest.raises(ValueError):
            Form5Dot7ShearBucklingResistance(h=500, t_f=600, t_w=10, f_bv=300, gamma_m_0=1.1)

    @pytest.mark.parametrize(
        ("representation", "expected"),
        [
            (
                "complete",
                r"V_{b,Rd} = \frac{\left(h - t_f \right) t_w f_{bv}}{\gamma_{M0}} = \frac{(500.00 - 20.00) \cdot 10 \cdot 300.00}{1.1} = 1309.09",
            ),
            ("short", r"V_{b,Rd} = 1309.09"),
            (
                "string",
                r"V_{b,Rd} = \frac{\left(h - t_f \right) t_w f_{bv}}{\gamma_{M0}} = \frac{(500.00 - 20.00) \cdot 10 \cdot 300.00}{1.1} = 1309.09",
            ),
        ],
    )
    def test_latex_output(self, representation: str, expected: str) -> None:
        """Test the latex implementation."""
        h = 500  # MM
        t_f = 20  # MM
        t_w = 10  # MM
        f_bv = 300  # MPA
        gamma_m_0 = 1.1

        form = Form5Dot7ShearBucklingResistance(
            h=h,
            t_f=t_f,
            t_w=t_w,
            f_bv=f_bv,
            gamma_m_0=gamma_m_0,
        ).latex()

        actual = {
            "complete": form.complete,
            "short": form.short,
            "string": str(form),
        }

        assert actual[representation] == expected, f"{representation} representation failed."



"""Testing formula 4.5N of EN 1992-1-1:2004."""

import pytest

from blueprints.codes.eurocode.en_1992_1_1_2004.chapter_4_durability_and_cover.table_4_1 import (
    Carbonation,
    Chemical,
    Chloride,
    ChlorideSeawater,
    FreezeThaw,
    Table4Dot1ExposureClasses,
)
from blueprints.codes.eurocode.en_1992_1_1_2004.chapter_4_durability_and_cover.table_4_3 import (
    ConcreteMaterial,
    ConcreteStrengthClass,
    Table4Dot3ConcreteStructuralClass,
)
from blueprints.codes.eurocode.en_1992_1_1_2004.chapter_4_durability_and_cover.table_4_5n import (
    Table4Dot5nMinimumCoverDurabilityPrestressingSteel,
)
from blueprints.type_alias import MM


class TestTable4Dot5nMinimumCoverDurabilityPrestressingSteel:
    """Validation for table 4.5N from EN 1992-1-1:2004."""

    @pytest.mark.parametrize(
        ("exposure_classes", "structural_class", "expected_result"),
        [
            (
                Table4Dot1ExposureClasses(Carbonation.XC2, Chloride.XD1, ChlorideSeawater.XS3, FreezeThaw.XF4, Chemical.XA1),
                6,
                55,
            ),
            (
                Table4Dot1ExposureClasses(Carbonation.XC2, Chloride.XD3, ChlorideSeawater.XS2, FreezeThaw.XF4, Chemical.XA1),
                5,
                50,
            ),
            (
                Table4Dot1ExposureClasses(Carbonation.XC2, Chloride.XD1, ChlorideSeawater.XS2, FreezeThaw.XF4, Chemical.XA1),
                5,
                50,
            ),
            (
                Table4Dot1ExposureClasses(Carbonation.NA, Chloride.NA, ChlorideSeawater.NA, FreezeThaw.NA, Chemical.NA),
                4,
                10,
            ),
            (
                Table4Dot1ExposureClasses(Carbonation.XC2, Chloride.NA, ChlorideSeawater.NA, FreezeThaw.XF4, Chemical.XA1),
                3,
                25,
            ),
            (
                Table4Dot1ExposureClasses(Carbonation.XC2, Chloride.XD1, ChlorideSeawater.NA, FreezeThaw.XF4, Chemical.XA1),
                2,
                30,
            ),
            (
                Table4Dot1ExposureClasses(Carbonation.XC4, Chloride.NA, ChlorideSeawater.NA, FreezeThaw.XF4, Chemical.XA1),
                1,
                20,
            ),
            (
                Table4Dot1ExposureClasses(Carbonation.XC1, Chloride.NA, ChlorideSeawater.NA, FreezeThaw.XF3, Chemical.XA1),
                5,
                25,
            ),
        ],
    )
    def test_evaluation(
        self, exposure_classes: Table4Dot1ExposureClasses, structural_class: Table4Dot3ConcreteStructuralClass, expected_result: MM
    ) -> None:
        """Test the evaluation of the result."""
        table_4_5 = Table4Dot5nMinimumCoverDurabilityPrestressingSteel(
            exposure_classes=exposure_classes,
            structural_class=structural_class,
        )

        assert table_4_5 == pytest.approx(expected=expected_result, rel=1e-4)

    def test_evaluation_valid_structural_class_type(self) -> None:
        """Test if the evaluation raises TypeError when the structural class is not an integer."""
        exposure_classes = Table4Dot1ExposureClasses(Carbonation.XC2, Chloride.XD1, ChlorideSeawater.XS3, FreezeThaw.XF4, Chemical.XA1)
        structural_class = Table4Dot3ConcreteStructuralClass(exposure_classes, 50, ConcreteMaterial(ConcreteStrengthClass("C20/25")), True, False)

        assert Table4Dot5nMinimumCoverDurabilityPrestressingSteel(exposure_classes, structural_class) == 40

    def test_evaluation_invalid_structural_class_type(self) -> None:
        """Test if the evaluation raises TypeError when the structural class is not an integer."""
        exposure_classes = Table4Dot1ExposureClasses(Carbonation.XC2, Chloride.XD1, ChlorideSeawater.XS3, FreezeThaw.XF4, Chemical.XA1)
        structural_class = "S4"

        with pytest.raises(TypeError):
            Table4Dot5nMinimumCoverDurabilityPrestressingSteel(exposure_classes, structural_class)  # type: ignore[arg-type]

    @pytest.mark.parametrize(
        ("representation", "expected_result"),
        [
            ("complete", r"c_{min,dur} = structural class S3 & exposure classes (XC2, XF4, XA1) = 25"),
            ("short", "c_{min,dur} = 25"),
        ],
    )
    def test_latex(self, representation: str, expected_result: str) -> None:
        """Test the latex implementation."""
        # Test data
        exposure_classes = Table4Dot1ExposureClasses(Carbonation.XC2, Chloride.NA, ChlorideSeawater.NA, FreezeThaw.XF4, Chemical.XA1)
        structural_class = Table4Dot3ConcreteStructuralClass(exposure_classes, 50, ConcreteMaterial(ConcreteStrengthClass("C20/25")), True, False)

        c_min_dur = Table4Dot5nMinimumCoverDurabilityPrestressingSteel(
            exposure_classes=exposure_classes,
            structural_class=structural_class,
        ).latex()

        actual = {
            "complete": c_min_dur.complete,
            "short": c_min_dur.short,
            "string": str(c_min_dur),
        }

        assert actual[representation] == expected_result, f"{representation} representation failed."



"""Testing formula 6.2 of EN 1993-1-1:2005."""

import pytest

from blueprints.codes.eurocode.en_1993_1_1_2005.chapter_6_ultimate_limit_state.formula_6_2 import Form6Dot2UtilizationRatio
from blueprints.validations import LessOrEqualToZeroError, NegativeValueError


class TestForm6Dot2UtilizationRatio:
    """Validation for form 6.2 from EN 1993-1-1:2005."""

    def test_evaluation(self) -> None:
        """Test the evaluation of the result."""
        # example values
        n_ed = 5.0  # [-]
        n_rd = 10.0  # [-]
        m_y_ed = 4.0  # [-]
        m_y_rd = 20.0  # [-]
        m_z_ed = 3.0  # [-]
        m_z_rd = 30.0  # [-]

        form_6_2 = Form6Dot2UtilizationRatio(n_ed=n_ed, n_rd=n_rd, m_y_ed=m_y_ed, m_y_rd=m_y_rd, m_z_ed=m_z_ed, m_z_rd=m_z_rd)
        # manually calculated result
        manually_calculated_result = (5 / 10) + (4 / 20) + (3 / 30)  # -

        assert form_6_2 == pytest.approx(manually_calculated_result, rel=1e-9)

    @pytest.mark.parametrize(
        ("n_ed", "n_rd", "m_y_ed", "m_y_rd", "m_z_ed", "m_z_rd"),
        [
            (-5.0, 10.0, 4.0, 20.0, 3.0, 30.0),
            (5.0, 10.0, -4.0, 20.0, 3.0, 30.0),
            (5.0, 10.0, 4.0, 20.0, -3.0, 30.0),
        ],
    )
    def test_raise_error_if_negative(self, n_ed: float, n_rd: float, m_y_ed: float, m_y_rd: float, m_z_ed: float, m_z_rd: float) -> None:
        """Test that a NegativeValueError is raised when a negative value is passed for n_ed."""
        with pytest.raises(NegativeValueError):
            Form6Dot2UtilizationRatio(n_ed=n_ed, n_rd=n_rd, m_y_ed=m_y_ed, m_y_rd=m_y_rd, m_z_ed=m_z_ed, m_z_rd=m_z_rd)

    @pytest.mark.parametrize(
        ("n_ed", "n_rd", "m_y_ed", "m_y_rd", "m_z_ed", "m_z_rd"),
        [
            (5.0, 0.0, 4.0, 20.0, 3.0, 30.0),
            (5.0, -10.0, 4.0, 20.0, 3.0, 30.0),
            (5.0, 10.0, 4.0, 0.0, 3.0, 30.0),
            (5.0, 10.0, 4.0, -20.0, 3.0, 30.0),
            (5.0, 10.0, 4.0, 20.0, 3.0, 0.0),
            (5.0, 10.0, 4.0, 20.0, 3.0, -30.0),
        ],
    )
    def test_raise_error_if_negative_or_zero(self, n_ed: float, n_rd: float, m_y_ed: float, m_y_rd: float, m_z_ed: float, m_z_rd: float) -> None:
        """Test that a LessOrEqualToZeroError is raised when a zero or negative value is passed for n_rd, m_y_rd, or m_z_rd."""
        with pytest.raises(LessOrEqualToZeroError):
            Form6Dot2UtilizationRatio(n_ed=n_ed, n_rd=n_rd, m_y_ed=m_y_ed, m_y_rd=m_y_rd, m_z_ed=m_z_ed, m_z_rd=m_z_rd)

    @pytest.mark.parametrize(
        ("representation", "expected"),
        [
            (
                "complete",
                (
                    r"UC = \frac{N_{Ed}}{N_{Rd}} + \frac{M_{y,Ed}}{M_{y,Rd}} + \frac{M_{z,Ed}}{M_{z,Rd}} ="
                    r" \frac{5.000}{10.000} + \frac{4.000}{20.000} + \frac{3.000}{30.000} = 0.800"
                ),
            ),
            ("short", r"UC = 0.800"),
        ],
    )
    def test_latex(self, representation: str, expected: str) -> None:
        """Test the latex representation of the form."""
        # Example values
        n_ed = 5.0  # [-]
        n_rd = 10.0  # [-]
        m_y_ed = 4.0  # [-]
        m_y_rd = 20.0  # [-]
        m_z_ed = 3.0  # [-]
        m_z_rd = 30.0  # [-]

        # Object to test
        form_6_2_latex = Form6Dot2UtilizationRatio(n_ed=n_ed, n_rd=n_rd, m_y_ed=m_y_ed, m_y_rd=m_y_rd, m_z_ed=m_z_ed, m_z_rd=m_z_rd).latex()

        actual = {
            "complete": form_6_2_latex.complete,
            "short": form_6_2_latex.short,
        }

        assert actual[representation] == expected, f"{representation} representation failed."



"""Formula 6.34 from EN 1993-1-1:2005: Chapter 6 - Ultimate Limit State."""

from blueprints.codes.eurocode.en_1993_1_1_2005 import EN_1993_1_1_2005
from blueprints.codes.formula import Formula
from blueprints.codes.latex_formula import LatexFormula, latex_replace_symbols
from blueprints.type_alias import DIMENSIONLESS, MM, MPA, N
from blueprints.validations import raise_if_less_or_equal_to_zero, raise_if_negative


class Form6Dot34CheckAxialForceY(Formula):
    r"""Class representing formula 6.34 for checking axial force about the y-y axis."""

    label = "6.34"
    source_document = EN_1993_1_1_2005

    def __init__(
        self,
        n_ed: N,
        h_w: MM,
        t_w: MM,
        f_y: MPA,
        gamma_m0: DIMENSIONLESS,
    ) -> None:
        r"""For doubly symmetrical I- and H-sections or other flanges sections,
        allowance need not be made for the effect of the axial force on the
        plastic resistance moment about the y-y axis when 6.33 and 6.34 are satisfied.

        EN 1993-1-1:2005 art.6.2.9(4) - Formula (6.34)

        Parameters
        ----------
        n_ed : N
            [$N_{Ed}$] Design axial force [$N$].
        h_w : MM
            [$h_w$] Web height [$mm$].
        t_w : MM
            [$t_{w}$] Web thickness [$mm$].
        f_y : MPA
            [$f_{y}$] Yield strength [$MPa$].
        gamma_m0 : DIMENSIONLESS
            [$\gamma_{M0}$] Partial safety factor [-].
        """
        super().__init__()
        self.n_ed = n_ed
        self.h_w = h_w
        self.t_w = t_w
        self.f_y = f_y
        self.gamma_m0 = gamma_m0

    @staticmethod
    def _evaluate(
        n_ed: N,
        h_w: MM,
        t_w: MM,
        f_y: MPA,
        gamma_m0: DIMENSIONLESS,
    ) -> bool:
        """Evaluates the formula, for more information see the __init__ method."""
        raise_if_negative(h_w=h_w, t_w=t_w, f_y=f_y, gamma_m0=gamma_m0)
        raise_if_less_or_equal_to_zero(gamma_m0=gamma_m0)

        return n_ed <= (0.5 * h_w * t_w * f_y) / gamma_m0

    def latex(self, n: int = 3) -> LatexFormula:
        """Returns LatexFormula object for formula 6.34."""
        _equation: str = r"N_{Ed} \leq \frac{0.5 \cdot h_{w} \cdot t_{w} \cdot f_{y}}{\gamma_{M0}}"
        _numeric_equation: str = latex_replace_symbols(
            _equation,
            {
                r"N_{Ed}": f"{self.n_ed:.{n}f}",
                r"h_{w}": f"{self.h_w:.{n}f}",
                r"t_{w}": f"{self.t_w:.{n}f}",
                r"f_{y}": f"{self.f_y:.{n}f}",
                r"\gamma_{M0}": f"{self.gamma_m0:.{n}f}",
            },
            False,
        )
        _numeric_equation_with_units: str = latex_replace_symbols(
            _equation,
            {
                r"N_{Ed}": rf"{self.n_ed:.{n}f} \ N",
                r"h_{w}": rf"{self.h_w:.{n}f} \ mm",
                r"t_{w}": rf"{self.t_w:.{n}f} \ mm",
                r"f_{y}": rf"{self.f_y:.{n}f} \ MPa",
                r"\gamma_{M0}": rf"{self.gamma_m0:.{n}f}",
            },
            True,
        )
        return LatexFormula(
            return_symbol="CHECK",
            result="OK" if self.__bool__() else "\\text{Not OK}",
            equation=_equation,
            numeric_equation=_numeric_equation,
            numeric_equation_with_units=_numeric_equation_with_units,
            comparison_operator_label="\\to",
            unit="",
        )



"""Testing formula 6.47 of EN 1992-1-1:2004."""

import pytest

from blueprints.codes.eurocode.en_1992_1_1_2004.chapter_6_ultimate_limit_state.formula_6_47_subs import (
    SubForm6Dot47FactorK,
    SubForm6Dot47FactorRhoL,
    SubForm6Dot47FactorSigmaCp,
    SubForm6Dot47FactorSigmaCy,
    SubForm6Dot47FactorSigmaCz,
)
from blueprints.validations import LessOrEqualToZeroError, NegativeValueError


class TestSubForm6Dot47FactorK:
    """Validation for sub-formula 1 of formula 6.47 from EN 1992-1-1:2004."""

    def test_evaluation(self) -> None:
        """Tests the evaluation of the result."""
        # Example values
        d = 500.0

        # Object to test
        formula = SubForm6Dot47FactorK(d=d)

        # Expected result, manually calculated
        manually_calculated_result = 1.632455532033676  # dimensionless

        assert formula == pytest.approx(expected=manually_calculated_result, rel=1e-4)

    @pytest.mark.parametrize(
        ("d"),
        [
            (-500.0),  # d is negative
            (0.0),  # d is zero
        ],
    )
    def test_raise_error_when_invalid_values_are_given(self, d: float) -> None:
        """Test invalid values."""
        with pytest.raises(LessOrEqualToZeroError):
            SubForm6Dot47FactorK(d=d)

    @pytest.mark.parametrize(
        ("representation", "expected"),
        [
            (
                "complete",
                r"k = \min \left( 1 + \sqrt{\frac{200}{d}}, 2.0 \right) = "
                r"\min \left( 1 + \sqrt{\frac{200}{500.000}}, 2.0 \right) = 1.632 \ -",
            ),
            ("short", r"k = 1.632 \ -"),
        ],
    )
    def test_latex(self, representation: str, expected: str) -> None:
        """Test the latex representation of the formula."""
        # Example values
        d = 500.0

        # Object to test
        latex = SubForm6Dot47FactorK(d=d).latex()

        actual = {
            "complete": latex.complete,
            "short": latex.short,
        }

        assert expected == actual[representation], f"{representation} representation failed."


class TestSubForm6Dot47FactorRhoL:
    """Validation for sub-formula 2 of formula 6.47 from EN 1992-1-1:2004."""

    def test_evaluation(self) -> None:
        """Tests the evaluation of the result."""
        # Example values
        rho_ly = 0.02
        rho_lz = 0.03

        # Object to test
        formula = SubForm6Dot47FactorRhoL(rho_ly=rho_ly, rho_lz=rho_lz)

        # Expected result, manually calculated
        manually_calculated_result = 0.02  # dimensionless

        assert formula == pytest.approx(expected=manually_calculated_result, rel=1e-4)

    @pytest.mark.parametrize(
        ("rho_ly", "rho_lz"),
        [
            (-0.02, 0.03),  # rho_ly is negative
            (0.02, -0.03),  # rho_lz is negative
        ],
    )
    def test_raise_error_when_invalid_values_are_given(self, rho_ly: float, rho_lz: float) -> None:
        """Test invalid values."""
        with pytest.raises(NegativeValueError):
            SubForm6Dot47FactorRhoL(rho_ly=rho_ly, rho_lz=rho_lz)

    @pytest.mark.parametrize(
        ("representation", "expected"),
        [
            (
                "complete",
                r"\rho_l = \min \left( \sqrt{\rho_{ly} \cdot \rho_{lz}}, 0.02 \right) = "
                r"\min \left( \sqrt{0.020 \cdot 0.030}, 0.02 \right) = 0.020 \ -",
            ),
            ("short", r"\rho_l = 0.020 \ -"),
        ],
    )
    def test_latex(self, representation: str, expected: str) -> None:
        """Test the latex representation of the formula."""
        # Example values
        rho_ly = 0.02
        rho_lz = 0.03

        # Object to test
        latex = SubForm6Dot47FactorRhoL(rho_ly=rho_ly, rho_lz=rho_lz).latex()

        actual = {
            "complete": latex.complete,
            "short": latex.short,
        }

        assert expected == actual[representation], f"{representation} representation failed."


class TestSubForm6Dot47FactorSigmaCp:
    """Validation for sub-formula 3 of formula 6.47 from EN 1992-1-1:2004."""

    def test_evaluation(self) -> None:
        """Tests the evaluation of the result."""
        # Example values
        sigma_cy = 1.0
        sigma_cz = 2.0

        # Object to test
        formula = SubForm6Dot47FactorSigmaCp(sigma_cy=sigma_cy, sigma_cz=sigma_cz)

        # Expected result, manually calculated
        manually_calculated_result = 1.5  # MPa

        assert formula == pytest.approx(expected=manually_calculated_result, rel=1e-4)

    @pytest.mark.parametrize(
        ("sigma_cy", "sigma_cz"),
        [
            (-1.0, 2.0),  # sigma_cy is negative
            (1.0, -2.0),  # sigma_cz is negative
        ],
    )
    def test_raise_error_when_invalid_values_are_given(self, sigma_cy: float, sigma_cz: float) -> None:
        """Test invalid values."""
        with pytest.raises(NegativeValueError):
            SubForm6Dot47FactorSigmaCp(sigma_cy=sigma_cy, sigma_cz=sigma_cz)

    @pytest.mark.parametrize(
        ("representation", "expected"),
        [
            (
                "complete",
                r"\sigma_{cp} = \frac{\sigma_{cy} + \sigma_{cz}}{2} = " r"\frac{1.000 + 2.000}{2} = 1.500 \ MPa",
            ),
            ("short", r"\sigma_{cp} = 1.500 \ MPa"),
        ],
    )
    def test_latex(self, representation: str, expected: str) -> None:
        """Test the latex representation of the formula."""
        # Example values
        sigma_cy = 1.0
        sigma_cz = 2.0

        # Object to test
        latex = SubForm6Dot47FactorSigmaCp(sigma_cy=sigma_cy, sigma_cz=sigma_cz).latex()

        actual = {
            "complete": latex.complete,
            "short": latex.short,
        }

        assert expected == actual[representation], f"{representation} representation failed."


class TestSubForm6Dot47FactorSigmaCy:
    """Validation for sub-formula 4 of formula 6.47 from EN 1992-1-1:2004."""

    def test_evaluation(self) -> None:
        """Tests the evaluation of the result."""
        # Example values
        n_ed_y = 1000.0
        a_cy = 500.0

        # Object to test
        formula = SubForm6Dot47FactorSigmaCy(n_ed_y=n_ed_y, a_cy=a_cy)

        # Expected result, manually calculated
        manually_calculated_result = 2.0  # MPa

        assert formula == pytest.approx(expected=manually_calculated_result, rel=1e-4)

    @pytest.mark.parametrize(
        ("n_ed_y", "a_cy"),
        [
            (-1000.0, 500.0),  # n_ed_y is negative
            (1000.0, -500.0),  # a_cy is negative
            (1000.0, 0.0),  # a_cy is zero
        ],
    )
    def test_raise_error_when_invalid_values_are_given(self, n_ed_y: float, a_cy: float) -> None:
        """Test invalid values."""
        with pytest.raises((NegativeValueError, LessOrEqualToZeroError)):
            SubForm6Dot47FactorSigmaCy(n_ed_y=n_ed_y, a_cy=a_cy)

    @pytest.mark.parametrize(
        ("representation", "expected"),
        [
            (
                "complete",
                r"\sigma_{cy} = \frac{N_{Ed,y}}{A_{cy}} = " r"\frac{1000.000}{500.000} = 2.000 \ MPa",
            ),
            ("short", r"\sigma_{cy} = 2.000 \ MPa"),
        ],
    )
    def test_latex(self, representation: str, expected: str) -> None:
        """Test the latex representation of the formula."""
        # Example values
        n_ed_y = 1000.0
        a_cy = 500.0

        # Object to test
        latex = SubForm6Dot47FactorSigmaCy(n_ed_y=n_ed_y, a_cy=a_cy).latex()

        actual = {
            "complete": latex.complete,
            "short": latex.short,
        }

        assert expected == actual[representation], f"{representation} representation failed."


class TestSubForm6Dot47FactorSigmaCz:
    """Validation for sub-formula 5 of formula 6.47 from EN 1992-1-1:2004."""

    def test_evaluation(self) -> None:
        """Tests the evaluation of the result."""
        # Example values
        n_ed_z = 2000.0
        a_cz = 1000.0

        # Object to test
        formula = SubForm6Dot47FactorSigmaCz(n_ed_z=n_ed_z, a_cz=a_cz)

        # Expected result, manually calculated
        manually_calculated_result = 2.0  # MPa

        assert formula == pytest.approx(expected=manually_calculated_result, rel=1e-4)

    @pytest.mark.parametrize(
        ("n_ed_z", "a_cz"),
        [
            (-2000.0, 1000.0),  # n_ed_z is negative
            (2000.0, -1000.0),  # a_cz is negative
            (2000.0, 0.0),  # a_cz is zero
        ],
    )
    def test_raise_error_when_invalid_values_are_given(self, n_ed_z: float, a_cz: float) -> None:
        """Test invalid values."""
        with pytest.raises((NegativeValueError, LessOrEqualToZeroError)):
            SubForm6Dot47FactorSigmaCz(n_ed_z=n_ed_z, a_cz=a_cz)

    @pytest.mark.parametrize(
        ("representation", "expected"),
        [
            (
                "complete",
                r"\sigma_{cz} = \frac{N_{Ed,z}}{A_{cz}} = " r"\frac{2000.000}{1000.000} = 2.000 \ MPa",
            ),
            ("short", r"\sigma_{cz} = 2.000 \ MPa"),
        ],
    )
    def test_latex(self, representation: str, expected: str) -> None:
        """Test the latex representation of the formula."""
        # Example values
        n_ed_z = 2000.0
        a_cz = 1000.0

        # Object to test
        latex = SubForm6Dot47FactorSigmaCz(n_ed_z=n_ed_z, a_cz=a_cz).latex()

        actual = {
            "complete": latex.complete,
            "short": latex.short,
        }

        assert expected == actual[representation], f"{representation} representation failed."
